/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

/**
 * Manages browser's right-click context menu additons
 */
EbayCompanion.BrowserContextMenu = {
  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      this._contextMenu = document.getElementById("contentAreaContextMenu");

      // This will be automatically removed by the EbayCompanion object
      EbayCompanion.addManagedEventListener(
        this._contextMenu, "popupshowing",
        let (that = this) function(event) that._onPopupShowing(event),
        false);

    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Hides or unhides our added menu items according to the context
   */
  _onPopupShowing : function(event) {
    try {
      if (gContextMenu) {

        if (gContextMenu.isTextSelected) {
          let stringBundle = EbayCompanion.Constants.stringBundle;
          let formattedString =
            stringBundle.getString("context.menu.search",
                                   [this._getTrimmedSelection()]);

          gContextMenu.
            setItemAttr("ebayCompanionContextSearch", "label", formattedString);
        }

        gContextMenu.
          showItem("ebayCompanionContextSearch", gContextMenu.isTextSelected);
      }
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }
  },

  /**
   * Gets the selected text and trims the selection if it's greater
   * than 15 chars.
   */
  _getTrimmedSelection : function() {
    let selectedText = "";

    try {
      selectedText = getBrowserSelection(16);
      if (!selectedText) {
        selectedText = "";
      } else if (selectedText.length > 15) {
        selectedText = selectedText.substr(0, 15) + "...";
      }
    } catch (e) {
      EbayCompanion.Logger.exception(e, EbayCompanion.Logger.DUMP_STACK);
    }

    return selectedText;
  }
}
