/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

/**
 * Main object for the eBay Companion Options Panel
 */
var FeedbackPane = {
  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      Cu.import("resource://ebaycompanion/common.js");
      Cu.import("resource://ebaycompanion/helpers/logger.js");
      Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
      Cu.import("resource://ebaycompanion/constants.js");
      Cu.import("resource://ebaycompanion/datasource.js");
      Cu.import("resource://ebaycompanion/helpers/observers.js");

      Observers.add(this, Topics.ACCOUNT_LOGGED_IN);

      var that = this;
      var activeAccount = Datasource.activeAccount();

      if (activeAccount) {
        var email = activeAccount.get("email");
        var userId = activeAccount.get("userId");
        var additionalFields = {};
        additionalFields["firefox_version"] = navigator.userAgent;

        // here we perform the actions related to the installed version of the
        // extension
        var getVersionCallback = function(aVersion) {
          additionalFields["extension_version"] = aVersion;
          var url =
              GSFastPass.generateURL(email, userId, userId, false, additionalFields);
          that._addGSJSFiles("fastpass", url);
        }
        UtilityHelper.getVersion(getVersionCallback);
      }
    }
    catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Uninitializes the resource.
   */
  uninit : function() {
    Observers.removeAll(this);
  },

  _addGSJSFiles : function(aJSId, aUrl) {
    try {
      var head = document.getElementsByTagName("head")[0];
      var script = document.createElement('script');
      script.id = aJSId;
      script.type = 'text/javascript';
      script.src = aUrl;
      head.appendChild(script);
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    if (Topics.ACCOUNT_LOGGED_IN == aTopic) {
      window.location.reload();
    }
  }
}

window.addEventListener("load", function(){ FeedbackPane.init(); }, false);
window.addEventListener("unload", function(){ FeedbackPane.uninit(); }, false);
