/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */
const HTMLNS = "http://www.w3.org/1999/xhtml";
/**
 * Manages user recent items
 */
Sidebar.TabRecent = {
  _sectionIndex: -1,

  GENERAL_ALERTS : "alerts",
  GENERAL_BIDS : "bids",
  GENERAL_SEARCHES : "searches",
  GENERAL_WATCHES : "watches",
  GENERAL_VIEWS : "views",

  init : function() {
    EbayCompanion.Observers.add(this, EbayCompanion.Topics.ACCOUNT_LOGGED_IN);
    EbayCompanion.Observers.add(this, EbayCompanion.Topics.ACCOUNT_LOGGED_OUT);
    EbayCompanion.Observers.add(this, EbayCompanion.Topics.UPDATE_FINISHED);
    EbayCompanion.Observers.add(this,
      EbayCompanion.Topics.RECENT_ALERTS_UPDATED);
    EbayCompanion.Observers.add(this,
      EbayCompanion.Topics.RECENT_BIDS_UPDATED);
    EbayCompanion.Observers.add(this,
      EbayCompanion.Topics.RECENT_SEARCHES_UPDATED);
    EbayCompanion.Observers.add(this,
      EbayCompanion.Topics.RECENT_VIEWS_UPDATED);
    EbayCompanion.Observers.add(this,
      EbayCompanion.Topics.RECENT_WATCHES_UPDATED);
    EbayCompanion.Observers.add(this,
      EbayCompanion.Topics.ITEM_TIME_ADJUSTED);

    this._setClearAllLink(this.GENERAL_ALERTS);
    this._setClearAllLink(this.GENERAL_SEARCHES);
    this._setSignInState(EbayCompanion.Datasource.activeAccount());
  },

  /**
   * Shows the sign in state.
   * @param aAccount object.
   */
  _setSignInState : function(aAccount) {
    if (aAccount) {
      this._showRecentAlerts();
      this._showRecentBids();
      this._showRecentSearches();
      //this._showRecentViews();
      this._showRecentWatches();
    }
  },

  /**
   * Sets the "Clear all" link for the specified section
   * @param aItemType the type of the item (recent alerts, recent bids, etc).
   */
  _setClearAllLink: function(aItemType) {
    let that = this;
    let clearAllLink =
      document.getElementsByClassName("recent-tab-clear-recent-" + aItemType);
    if (clearAllLink && clearAllLink.length > 0) {
      for (let i = 0; i < clearAllLink.length; i++) {
        clearAllLink[i].addEventListener("click", function(aEvent) {
          switch (aItemType) {
            case that.GENERAL_ALERTS:
              that._clearRecentAlerts();
              break;
            case that.GENERAL_SEARCHES:
              that._clearRecentSearches();
              break;
          }
        }, false);
      }
    }
  },

  /**
   * Clears the child nodes of a recent XXX section.
   * @param aItemType the type of the item (recent alerts, recent bids, etc).
   */
  _clearRecentSection: function(aItemType) {
    let contentNode =
      document.getElementById("recent-tab-recent-" + aItemType + "-content");
    if (contentNode) {
      while (contentNode.hasChildNodes()) {
        contentNode.removeChild(contentNode.lastChild);
      }
    }
  },

  /**
   * Clears the recent alerts.
   */
  _clearRecentAlerts: function() {
    this._clearRecentSection(this.GENERAL_ALERTS);
    EbayCompanion.RecentAlertService.clearRecentAlerts(true);
  },

  /**
   * Shows the recent alerts.
   */
  _showRecentAlerts: function() {
    this._createCollapsibleSection(this.GENERAL_ALERTS);
  },

  /**
   * Creates a recent alert node element.
   * @param aRecentAlert the recent alert object.
   */
  _createRecentAlertElement: function(aRecentAlert) {
    try {
      let item;
      let itemId;
      let recentAlert;
      let recentAlertLink;
      let recentAlertState;
      let recentAlertStateText;
      let recentAlertDescription = "";
      let recentAlertElement;
      let separatorElement;
      let mostRecentWindow;

      recentAlertElement = document.createElementNS(HTMLNS, "li");
      recentAlertState = document.createElementNS(HTMLNS, "b");
      recentAlertLink = document.createElementNS(HTMLNS, "a");
      separatorElement = document.createElementNS(HTMLNS, "span");

      item = aRecentAlert.get("alert").object;
      itemId = item.get("itemId");
      if (item.get("title")) {
        recentAlertDescription = item.get("title");
      }
      recentAlertStateText = EbayCompanion.ItemService.getItemState(
        null, aRecentAlert.get("alertType"), null);

      if (recentAlertStateText.indexOf("(") !== -1) {
        recentAlertStateText =
          recentAlertStateText.substring(
            0, recentAlertStateText.indexOf("(") - 1);
      }

      // XXX: get the most recent window to avoid a "NS_ERROR_NOT_INITIALIZED"
      // error in the console.
      mostRecentWindow = EbayCompanion.UtilityHelper.getMostRecentWindow();
      // XXX: to show the tooltiptext
      mostRecentWindow.setTimeout(function() {
        recentAlertLink.setAttribute("tooltiptext", recentAlertDescription);
      }, 10);

      recentAlertLink.addEventListener("click", function(aEvent) {
        EbayCompanion.EventAnalytics.push(
          ['_trackEvent', "recent-notification", "clicked"]);
        EbayCompanion.openPage(
          aEvent, "recentAlertClick", "listing2", { "itemid":  itemId });
      }, false);

      recentAlertLink.appendChild(
        document.createTextNode(recentAlertDescription));
      recentAlertState.appendChild(
        document.createTextNode(recentAlertStateText));
      separatorElement.appendChild(document.createTextNode(" - "));
      recentAlertElement.appendChild(recentAlertState);
      recentAlertElement.appendChild(separatorElement);
      recentAlertElement.appendChild(recentAlertLink);
      //trim the title.
      recentAlertElement.className = recentAlertElement.className + " ellipsis";

      return recentAlertElement;
    } catch(e) {
        EbayCompanion.Logger.error(e);
        return null;
    }
  },

  /**
   * Clears the recent bids.
   */
  _clearRecentBids: function() {
    this._clearRecentSection(this.GENERAL_BIDS);
    EbayCompanion.RecentBidService.clearRecentBids(true);
  },

  /**
   * Shows the recent bids.
   */
  _showRecentBids: function() {
    this._createCollapsibleSection(this.GENERAL_BIDS);
  },

  /**
   * Clears the recent alerts.
   */
  _clearRecentSearches: function() {
    this._clearRecentSection(this.GENERAL_SEARCHES);
    EbayCompanion.RecentSearchService.clearRecentSearches(true);
  },

  /**
   * Shows the recent alerts.
   */
  _showRecentSearches: function() {
    this._createCollapsibleSection(this.GENERAL_SEARCHES);
  },

  /**
   * Creates a recent search node element.
   * @param aRecentSearch the recent search object.
   */
  _createRecentSearchElement: function(aRecentSearch) {
    try {
      let recentSearch;
      let recentSearchElement;
      let recentSearchLink;
      let searchString;

      recentSearchElement = document.createElementNS(HTMLNS, "li");
      recentSearchLink = document.createElementNS(HTMLNS, "a");
      searchString = aRecentSearch.get("searchString");
      recentSearchLink.appendChild(
        document.createTextNode(decodeURIComponent(searchString)));
      recentSearchLink.className = recentSearchLink.className + " link-block";

      recentSearchLink.addEventListener("click", function(aEvent) {
        EbayCompanion.EventAnalytics.push(
          ['_trackEvent', "recent-search", "clicked"]);
        EbayCompanion.openPage(
          aEvent, "recentSearchClick", "search", {query: searchString});
      }, false);

      recentSearchElement.appendChild(recentSearchLink);
      return recentSearchElement;
    } catch(e) {
        EbayCompanion.Logger.error(e);
        return null;
    }
  },

  /**
   * Clears the recent views.
   */
  _clearRecentViews: function() {
    this._clearRecentSection(this.GENERAL_VIEWS);
    EbayCompanion.RecentViewService.clearRecentViews(true);
  },

  /**
   * Shows the recent views.
   */
  _showRecentViews: function() {
    this._createCollapsibleSection(this.GENERAL_VIEWS);
  },

  /**
   * Clears the recent watches.
   */
  _clearRecentWatches: function() {
    this._clearRecentSection(this.GENERAL_WATCHES);
    EbayCompanion.RecentWatchService.clearRecentWatches(true);
  },

  /**
   * Shows the recent watches.
   */
  _showRecentWatches: function() {
    this._createCollapsibleSection(this.GENERAL_WATCHES);
  },

  /**
   * Updates the recent item time left information.
   * @param aItemType the type of the item (recent alerts, recent bids, etc).
   */
  _updateRecentItem: function(aItemType) {
    let elementsToRemove = new Array();
    let contentNode =
      document.getElementById("recent-tab-recent-" + aItemType + "-content");

    for (let i = 0; i < contentNode.childNodes.length; i++) {
      let item = contentNode.childNodes[i].item;
      if (item) {
        item = item.copy();
        if (!item.get("isEnded")) {
          let newEndTime = EbayCompanion.ItemService.getItemTimeLeft(item);
          //XXX: Item class for Google Chrome codebase does have the "timeLeft"
          //property. We'll leave this setting commented because the codebase
          //for Firefox does not have the property.
          //item.set("timeLeft", newEndTime);
          contentNode.childNodes[i].item = item;
          if (newEndTime != EbayCompanion.Constants.stringBundle.getString(
            "item.timeleft.ended.label")) {
            if (contentNode.childNodes[i].getElementsByTagName("p")[0]) {
              contentNode.childNodes[i].getElementsByTagName("p")[0].innerHTML =
                EbayCompanion.Constants.parseMarkupString(newEndTime);
              contentNode.childNodes[i].getElementsByTagName("p")[0].className =
                " item-status " + Sidebar.TabRecent._getItemStateStyle(item);
            }
          } else {
            //let's remove the item from the list, because we don't need it
            //anymore in the UI.
            elementsToRemove.push(item);
          }
        } else {
          //let's remove the item from the list, because we don't need it
          //anymore in the UI.
          elementsToRemove.push(item);
        }
      }
    }
    for (let i = 0; i < elementsToRemove.length; i++) {
      let elementToRemove = elementsToRemove[i];
      Sidebar.TabRecent._removeRecentItem(aItemType, elementToRemove);
    }
  },

  /**
   * Gets the items cache, depending on the item type.
   * @param aItemType the type of the item (recent alerts, recent bids, etc).
   */
  _getItemsCache: function(aItemType) {
    let itemsCache = [];

    switch (aItemType) {
      case this.GENERAL_ALERTS:
        itemsCache = EbayCompanion.RecentAlertService.recentAlerts;
        break;
      case this.GENERAL_BIDS:
        itemsCache = EbayCompanion.RecentBidService.recentBids;
        break;
      case this.GENERAL_SEARCHES:
        itemsCache = EbayCompanion.RecentSearchService.recentSearches;
        break;
      case this.GENERAL_VIEWS:
        itemsCache = EbayCompanion.RecentViewService.recentViews;
        break;
      case this.GENERAL_WATCHES:
        itemsCache = EbayCompanion.RecentWatchService.recentWatches;
        break;
    }

    return itemsCache;
  },

  /**
   * Creates a node element depending on the item type.
   * @param aItem the item to extract the information from.
   * @param aItemType the type of the item (recent alerts, recent bids, etc).
   */
  _createItemNode: function(aItem, aItemType) {
    let itemNode;

    switch (aItemType) {
      case this.GENERAL_ALERTS:
        itemNode = this._createRecentAlertElement(aItem);
        break;
      case this.GENERAL_SEARCHES:
        itemNode = this._createRecentSearchElement(aItem);
        break;
      case this.GENERAL_VIEWS:
        itemNode = this._createRecentViewElement(aItem);
        break;
      case this.GENERAL_BIDS:
      case this.GENERAL_WATCHES:
        itemNode = this._createRecentElement(aItem, aItemType);
        break;
    }

    return itemNode;
  },

  /**
   * Creates a collapsible section for recentXXX elements.
   * @param aItemType the item type (alerts, bids, etc.).
   */
  _createCollapsibleSection: function(aItemType) {
    let itemsQuantity = 0;
    let itemsAdded = 0;
    let itemsCache = this._getItemsCache(aItemType);
    let contentNode = document.getElementById(
      "recent-tab-recent-" + aItemType + "-content");
    let containerNode = document.getElementById(
      "recent-tab-recent-" + aItemType + "-container");
    let headerNode = document.getElementById(
      "recent-tab-recent-" + aItemType +"-header");
    let itemNode;
    let panelState;
    let expandHeader = false;
    let sectionIndex;
    let currentIndex = -1;

    sectionIndex = headerNode.getAttribute("section-index");
    if (sectionIndex) {
      currentIndex = sectionIndex;
    } else {
      currentIndex = ++this._sectionIndex;
    }
    headerNode.setAttribute("section-index", currentIndex);

    this._clearRecentSection(aItemType);

    if (itemsCache && itemsCache.length > 0) {
      itemsQuantity = itemsCache.length ;
    }

    if (itemsQuantity > 0) {
      for (let i = itemsQuantity - 1; i >= 0; i--) {
        itemNode = this._createItemNode(itemsCache[i], aItemType);
        if (itemNode) {
          contentNode.appendChild(itemNode);
          itemsAdded++;
        }
      }
    }

    let contentNodeChildren = contentNode.childNodes;
    if (itemsAdded == 0 && contentNodeChildren.length == 0) {
      itemNode = document.createElementNS(HTMLNS, "li");
      itemNode.className = itemNode.className + " emptySection";
      itemNode.appendChild(
        document.createTextNode(
          EbayCompanion.Constants.stringBundle.getString(
            "sidebar.no.recent." + aItemType)));
      contentNode.appendChild(itemNode);
    }

    //section header
    panelState =
      EbayCompanion.StateHelper.getAccordionStateForItem(currentIndex);

    //expand the container by default when there are > 0 items or if the
    //previous panel state was "expanded", by adding the "in" class to the
    //container div.
    if (panelState == 1 || (panelState == -1 && itemsAdded > 0)) {
      containerNode.className = containerNode.className + " in";
      expandHeader = true;
    } else {
      //remove the "in" class
      containerNode.className = containerNode.className.replace(/\bin\b/,'');
      expandHeader = false;
    }

    this._createCollapsibleSectionHeader(
      aItemType, itemsAdded, expandHeader);
  },

  /**
   * Creates a collapsible section header, with the toggle icons, the section
   * description ("Alerts", "Bids", etc.) and the items quantity for the section.
   * @param aItemType the item type (alerts, bids, etc.).
   * preference, like [1,0,1,0,1] "Alerts" will be the zero index, "Bids" will
   * be the 1 index and so on.
   * @param aItemsQuantity the amount of items of type recentXXX stored in
   * the cache.
   * @param aExpandHeader indicates if the header should be expanded or
   * collapsed.
   */
  _createCollapsibleSectionHeader: function(
      aItemType, aItemsQuantity, aExpandHeader) {
    let headerNode = document.getElementById(
      "recent-tab-recent-" + aItemType +"-header");
    let headerDescriptionNode = document.getElementById(
      "recent-tab-recent-" + aItemType + "-header").
        getElementsByClassName("recent-tab-header-description")[0];
    let quantityNode = document.getElementById(
      "recent-tab-recent-" + aItemType + "-header").
        getElementsByClassName("recent-tab-items-quantity")[0];

    headerDescriptionNode.innerHTML =
      EbayCompanion.Constants.parseMarkupString(
        EbayCompanion.Constants.stringBundle.getString(
          "sidebar.recent." + aItemType));
    quantityNode.innerHTML =
      EbayCompanion.Constants.parseMarkupString(
        this._getItemQuantityString(aItemsQuantity));

    //collapse the panel if there are no items to display.
    if (!aExpandHeader) {
      headerNode.className = headerNode.className + " collapsed";
    } else {
      headerNode.className = headerNode.className.replace(/\bcollapsed\b/,'');
    }

    headerNode.addEventListener("click", function(event) {
      let isCollapsed =
        event.currentTarget.className.indexOf("collapsed") > -1 ? true : false;
      let state = isCollapsed ? 1 : 0;
      EbayCompanion.StateHelper.setAccordionState(
        event.currentTarget.getAttribute("section-index"), state);
    }, false);
  },

  /**
   * Creates a recentXXX element node.
   * @param aItem the item object.
   * @param aItemType the type of the element to be created
   */
  _createRecentElement: function(aItem, aItemType) {
    try {
      let item;
      let itemNode;
      let itemId;
      let newEndTime = "";
      let element = null;
      let elementWrapper = null;
      let container;
      let imageContainer;
      let timeContainer;

      itemId = aItem.get("itemId");
      item = EbayCompanion.Datasource.getItem(itemId);

      if (item && !item.get("isEnded")) {
        newEndTime = EbayCompanion.ItemService.getItemTimeLeft(item);
        if (newEndTime != EbayCompanion.Constants.stringBundle.getString(
              "item.timeleft.ended.label")) {
          element = document.createElementNS(HTMLNS, "li");
          elementWrapper = document.createElement("box");
          container = document.createElementNS(HTMLNS, "div");
          imageContainer = document.createElementNS(HTMLNS, "div");
          timeContainer = document.createElementNS(HTMLNS, "p");

          element.setAttribute("item", item);
          element.item = item;
          container.className = container.className + " l-shad lftd thumb";
          timeContainer.className = " item-status " +
            this._getItemStateStyle(item);
          timeContainer.appendChild(document.createTextNode(newEndTime));
          imageContainer.className = imageContainer.className + " image";
          imageContainer.setAttribute("style",
            "background-image: url('" + this._getItemImage(item) + "')");
          imageContainer.appendChild(timeContainer);
          container.appendChild(imageContainer);
          if (item.get("title")) {
            /**
             * XXX: setting the title attribute to a li or a div was not
             * working, so we wrap the content in a XUL box and set the
             * tooltiptext attribute, which works OK in this case.
             */
            elementWrapper.setAttribute("tooltiptext", item.get("title"));
          }
          elementWrapper.appendChild(container);
          element.appendChild(elementWrapper);

          element.addEventListener("click", function(aEvent) {
            // select the respective item list tab first
            let tab = EbayCompanion.StateHelper.BUYING_TAB;
            let defaultView = EbayCompanion.Constants.prefBranch.get("default.view");

            // default view: active-ended = 0, buying-selling = 1
            switch (defaultView) {
              case 0:
                tab = EbayCompanion.StateHelper.ACTIVE_TAB;
                Sidebar._setFilterSelection("active", "all");
                break;
              case 1:
                tab = EbayCompanion.StateHelper.BUYING_TAB;
                Sidebar._setFilterSelection("buying", "all");
                break;
            }

            Sidebar.switchToTab(tab, true);
            itemList = document.getElementById("gs-ebay-sidebar-item-list");
            itemNode = itemList.findItem(item);
            if (itemNode) {
              // now display the item details
              EbayCompanion.ItemService.showItemDetails(itemNode);
            }
            let target = "recent-watch";
            if (aItemType == Sidebar.TabRecent.GENERAL_BIDS) {
              target = "recent-bid";
            }
            EbayCompanion.EventAnalytics.push(
              ['_trackEvent', target, "clicked"]);
          }, false);
        } else {
          this._removeRecentItem(aItemType, item);
        }
      } else if (item && item.get("isEnded")) {
        this._removeRecentItem(aItemType, item);
      }

      return element;
    } catch (e) {
      EbayCompanion.Logger.error(e);
      return null;
    }
  },

  /**
   * Creates a string representation for the items quantity for a recentXXX
   * section.
   * @param aItemQuantity the items quantity.
   */
  _getItemQuantityString: function(aItemQuantity) {
    let itemQuantity = 0;

    if (aItemQuantity && parseInt(aItemQuantity) > 0) {
      itemQuantity = aItemQuantity;
    }

    return "(" + itemQuantity + ")";
  },

  /**
   * Returns the URL for the item image.
   * @param aItem the item object.
   */
  _getItemImage: function(aItem) {
    let itemImageURL = "";

    if (aItem.get("thumbnailUrl")) {
      itemImageURL = aItem.get("thumbnailUrl");
    } else {
      if (aItem.get("imageUrl")) {
        itemImageURL = aItem.get("imageUrl");
      }
    }

    return itemImageURL;
  },

  /**
   * Returns the css style name for an item, depending on its time left.
   * @param aItem the item object.
   */
  _getItemStateStyle: function(aItem) {
    let state;
    let stateStyle = "";
    let endingSoon;

    state = aItem.getCurrentState(
      EbayCompanion.Datasource.activeAccount().get("userId"));
    endingSoon = EbayCompanion.ItemService.getIsItemEndingSoon(aItem);
    stateStyle = EbayCompanion.ItemService.getItemStateStyle(state, endingSoon);

    return stateStyle;
  },

  /**
   * Removes, from the database and memory, a recent item that has just ended.
   * @param aItemType the item type.
   * @param aItem the item data.
   */
  _removeRecentItem: function(aItemType, aItem) {
    switch (aItemType) {
      case this.GENERAL_ALERTS:
        break;
      case this.GENERAL_BIDS:
        EbayCompanion.RecentBidService.removeRecentBid(
          aItem.get("itemId"));
        break;
      case this.GENERAL_SEARCHES:
        break;
      case this.GENERAL_VIEWS:
        break;
      case this.GENERAL_WATCHES:
        EbayCompanion.RecentAlertService.removeRecentAlert(
          aItem.get("itemId"));
        EbayCompanion.RecentWatchService.removeRecentWatch(
          aItem.get("itemId"));
        break;
    }
  },

  observe : function(aSubject, aTopic, aData) {
    try {
      switch (aTopic) {
        case EbayCompanion.Topics.ACCOUNT_LOGGED_IN:
          this._setSignInState(aSubject.object);
          break;
        case EbayCompanion.Topics.ACCOUNT_LOGGED_OUT:
          this._setSignInState(null);
          break;
        case EbayCompanion.Topics.UPDATE_FINISHED:
          this._setSignInState(EbayCompanion.Datasource.activeAccount());
          break;
        case EbayCompanion.Topics.RECENT_ALERTS_UPDATED:
          this._showRecentAlerts();
          break;
        case EbayCompanion.Topics.RECENT_BIDS_UPDATED:
          this._showRecentBids();
          break;
        case EbayCompanion.Topics.RECENT_SEARCHES_UPDATED:
          this._showRecentSearches();
          break;
        case EbayCompanion.Topics.RECENT_VIEWS_UPDATED:
          //this._showRecentViews();
          break;
        case EbayCompanion.Topics.RECENT_WATCHES_UPDATED:
          this._showRecentWatches();
          break;
        case EbayCompanion.Topics.ITEM_TIME_ADJUSTED:
          this._updateRecentItem(this.GENERAL_BIDS);
          this._updateRecentItem(this.GENERAL_WATCHES);
          break;
      }
    } catch (ex) {
      // XXX: for Firefox 4 we get a "EbayCompanion is undefined" error.
      // There's no need to the anything else when catching that error.
    }
  }

}
