/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["DealsApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const DEALS_API_URL =
  "http://anywhere.ebay.com/services/feed/deals/?version=3&dealdetail=2&type=0";
  //"http://anywhere.ebay.com/services/feed/deals/?siteid=0&version=3&dealdetail=1&type=0&format=json";

/**
 * The Deals API.
 */
DealsApi = {
  /* Application Name. */
  _appName : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/constants.js");
    Cu.import("resource://ebaycompanion/datasource.js");
    Cu.import("resource://ebaycompanion/helpers/logger.js");
    Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
    Cu.import("resource://ebaycompanion/objects/deal.js");
  },

  /**
   * Extracts the response.
   * @param aResponseText the xml.
   * @return the body XML of the response.
   */
  _parseGetDailyDealsResponse : function(aResponseText) {
    let xmlResponse = null;

    // workaround for E4X bug in Firefox and remove namespaces.
    aResponseText = aResponseText.replace(/<\?xml .*?>/, "");
    aResponseText = aResponseText.replace(/xmlns:xsi=["'].*?["']/, "");
    aResponseText = aResponseText.replace(/xmlns=["'].*?["']/, "");
    aResponseText = aResponseText.replace(/xsi:schemaLocation=["'].*?["']/, "");

    var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
      .createInstance(Ci.nsIDOMParser);
    let xmlTree = parser.parseFromString(aResponseText,"text/xml");
    var xmlObject = ObjectHelper.getJXONTree(xmlTree);

    xmlObject = xmlObject["InitiativeFeed"];

    let ack = xmlObject["Ack"];

    let dealsArray = [];
    if ("Success" == ack) {
      // Extract items from the call response
      let xmlItems = xmlTree.getElementsByTagName("Item");
      let items = [];
      for (let i=0; i<xmlItems.length; i++) {
        let xmlItem = xmlItems[i];
        let jxonItem = ObjectHelper.getJXONTree(xmlItem);
        items.push(jxonItem);
      }
      dealsArray = this._buildDealsArray(items);
    } else {
      let message = xmlObject["Errors"]["LongMessage"];
      Logger.error("getDailyDeals: " + ack + ": " + message);
    }

    return dealsArray;

  },

  /**
   * Calls the daily deals service to get them
   */
  getDailyDeals : function(aCallback) {
    let that = this;
    let callback = function(aRequest) {
      if (aRequest) {
        let result = that._parseGetDailyDealsResponse(aRequest.responseText);
        try {
          if (aCallback) aCallback(result);
        }
        catch (e) {
          Logger.exception(e, Logger.DUMP_STACK)
        }
      }
    };

    this._sendRequest(callback);
  },

  /**
   * Builds the deals array.
   * @param aDealsListNode the node containing the daily deals
   */
  _buildDealsArray : function(aDealsListNode) {
    let dealsArray = new Array();
    // DEFINE A LIMIT TO ONLY PROCESS THE FIRST 7 DEALS
    const LIMIT = 7;

    let xmlItem;

    for (let i = 0; i < aDealsListNode.length && i < LIMIT; i++) {
      xmlItem = aDealsListNode[i];
      let deal = this._populateFromJXONObject(xmlItem);
      dealsArray.push(deal);
    }

    return dealsArray;
  },

  _populateFromJXONObject : function(aXMLNode) {
    let itemId = Number(aXMLNode["@ItemID"]);
    let deal = new Deal(itemId);
    deal.fromJXONObject(aXMLNode);

    return deal;
  },

  /**
   * Sends a request to the WS.
   * @param aCallback the request callback.
   */
  _sendRequest : function(aCallback) {
    let request =
      Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
        createInstance(Ci.nsIXMLHttpRequest);
    try {
    let url = DEALS_API_URL;
    let siteId = Constants.siteIdForSite(Datasource.homeSite());
    url += "&siteid=" + siteId;
    Logger.log("Retrieving Deals: " + url);

    request.mozBackgroundRequest = true;
    request.open("GET", url, true);
    request.onreadystatechange = function() {
      if (4 != request.readyState) { return; }
      aCallback(request);
    };
    request.send(null);

    } catch(e) {
      Logger.error(e);
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(DealsApi);
