/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["Constants"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

var Constants = {
  // max number of records to be kept in the trackedUrl table before setting
  // submitData flag to true (to flush data to eBay servers)
  TRACKED_URLS_COUNT_LIMIT : 1000,
  // max number of records to be kept in the trackedUrl table, if limit is
  //reached and a new url needs to be recorded, we clear the oldest record and
  //save the new one.
  STORAGE_TRACKED_URLS_LIMIT : 1500,

  //different formats for sending / getting information
  FORMAT_XML : "XML",
  FORMAT_NV : "NV",
  FORMAT_JSON : "JSON",
  FORMAT_SOAP : "SOAP",

  // Lookup table providing data for site Ids
  _siteData : {
    "Australia":      {num:  15, iso: "AU", cur: "AUD", symb: "$*",   dom: "ebay.com.au",  rid:  "705-51927-19398-0"},
    "Austria":        {num:  16, iso: "AT", cur: "EUR", symb: "€*",   dom: "ebay.at",      rid: "5221-51945-19398-0"},
    "Belgium.Dutch":  {num: 123, iso: "BE", cur: "EUR", symb: "€*",   dom: "benl.ebay.be", rid: "1553-51946-19398-0"},
    "Belgium.French": {num:  23, iso: "BE", cur: "EUR", symb: "€*",   dom: "befr.ebay.be", rid: "1553-51946-19398-0"},
    "Canada":         {num:   2, iso: "CA", cur: "CAD", symb: "$*",   dom: "ebay.ca",      rid:  "706-51947-19398-0"},
    "CanadaFrench":   {num: 210, iso: "CA", cur: "CAD", symb: "$*",   dom: "cafr.ebay.ca", rid:  "706-51947-19398-0"},
    "China":          {num: 223, iso: "CN", cur: "TWD", symb: "$*",   dom: "ebay.com.hk",  rid: "4080-71112-19398-2"},
    "France":         {num:  71, iso: "FR", cur: "EUR", symb: "€*",   dom: "ebay.fr",      rid:  "709-47295-17703-1"},
    "Germany":        {num:  77, iso: "DE", cur: "EUR", symb: "€*",   dom: "ebay.de",      rid:  "707-37276-17702-1"},
    "HongKong":       {num: 201, iso: "HK", cur: "HKD", symb: "$*",   dom: "ebay.com.hk",  rid: "3422-51948-19398-0"},
    "India":          {num: 203, iso: "IN", cur: "INR", symb: "Rs *", dom: "ebay.in",      rid: "4686-51949-19398-0"},
    "Ireland":        {num: 205, iso: "IE", cur: "EUR", symb: "€*",   dom: "ebay.ie",      rid: "5282-51950-19398-0"},
    "Italy":          {num: 101, iso: "IT", cur: "EUR", symb: "€*",   dom: "ebay.it",      rid:  "724-51951-19398-0"},
    "Malaysia":       {num: 207, iso: "MY", cur: "MYR", symb: "RM *", dom: "ebay.com.my",  rid: "4825-71113-19398-0"},
    "Netherlands":    {num: 146, iso: "NL", cur: "EUR", symb: "€*",   dom: "ebay.nl",      rid: "1346-51952-19398-0"},
    "Philippines":    {num: 211, iso: "PH", cur: "PHP", symb: "₱*",   dom: "ebay.ph",      rid: "4824-71114-19398-0"},
    "Poland":         {num: 212, iso: "PL", cur: "PLN", symb: "* zł", dom: "ebay.pl",      rid: "4908-51953-19398-0"},
    "Singapore":      {num: 216, iso: "SG", cur: "SNG", symb: "S$*",  dom: "ebay.com.sg",  rid: "3423-51954-19398-0"},
    "Spain":          {num: 186, iso: "ES", cur: "EUR", symb: "€*",   dom: "ebay.es",      rid: "1185-51955-19398-0"},
    "Switzerland":    {num: 193, iso: "CH", cur: "CHF", symb: "* Fr", dom: "ebay.ch",      rid: "5222-51956-19398-0"},
    "UK":             {num:   3, iso: "GB", cur: "GBP", symb: "£*",   dom: "ebay.co.uk",   rid:  "710-47297-17704-0"},
    "US":             {num:   0, iso: "US", cur: "USD", symb: "$*",   dom: "ebay.com",     rid:  "711-47294-18009-0"}
  },

  /* array to map locales and site data */
  _localeSiteDataSelector : {
    "EN-AU" : "Australia",
    "DE-AT" : "Austria",
    "NL-BE" : "Belgium.Dutch",
    "FR-BE" : "Belgium.French",
    "EN-CA" : "Canada",
    "FR-CA" : "CanadaFrench",
    "ZH-CN" : "China",
    "ZH-TW" : "HongKong",
    "FR"    : "France",
    "DE"    : "Germany",
    "HI-IN" : "India",
    "GA-IE" : "Ireland",
    "IT"    : "Italy",
    "NL"    : "Netherlands",
    "PH"    : "Philippines",
    "MY"    : "Malaysia",
    "PL"    : "Poland",
    "EN-SG" : "Singapore",
    "ES-ES" : "Spain",
    "DE-CH" : "Switzerland",
    "EN-GB" : "UK",
    "EN-US" : "US"
  },

  // Tags to be used to load installation tracking image
  _installTrackTags : {
    "EN-AU" : "705-51927-19398-3",
    "DE-AT" : "5221-51945-19398-3",
    "NL-BE" : "1553-51946-19398-3",
    "FR-BE" : "1553-51946-19398-3",
    "EN-CA" : "706-51947-19398-3",
    "FR-CA" : "706-51947-19398-3",
    "ZH-CN" : "4080-71112-19398-1",
    "ZH-TW" : "3422-51948-19398-3",
    "FR"    : "709-47295-17703-2",
    "DE"    : "707-37276-17702-0",
    "HI-IN" : "4686-51949-19398-2",
    "GA-IE" : "5282-51950-19398-3",
    "IT"    : "724-51951-19398-2",
    "NL"    : "1346-51952-20245-1",
    "PH"    : "4824-71114-19398-2",
    "MY"    : "4825-71113-19398-2",
    "PL"    : "4908-51953-19398-2",
    "EN-SG" : "3423-51954-19398-3",
    "ES-ES" : "1185-51955-19398-2",
    "DE-CH" : "5222-51956-19398-3",
    "EN-GB" : "710-47297-17704-1",
    "EN-US" : "711-47294-18009-1"
  },

  // Array to map global ids and site data
  _GLOBAL_ID_DATA : {
    "Australia"      : "EBAY-AU",
    "Austria"        : "EBAY-AT",
    "Belgium.Dutch"  : "EBAY-NLBE",
    "Belgium.French" : "EBAY-FRBE",
    "Canada"         : "EBAY-ENCA",
    "CanadaFrench"   : "EBAY-FRCA",
    "France"         : "EBAY-FR",
    "Germany"        : "EBAY-DE",
    "HongKong"       : "EBAY-HK",
    "India"          : "EBAY-IN",
    "Ireland"        : "EBAY-IE",
    "Italy"          : "EBAY-IT",
    "Netherlands"    : "EBAY-NL",
    "Philippines"    : "EBAY-PH",
    "Malaysia"       : "EBAY-MY",
    "Poland"         : "EBAY-PL",
    "Singapore"      : "EBAY-SG",
    "Spain"          : "EBAY-ES",
    "Switzerland"    : "EBAY-CH",
    "UK"             : "EBAY-GB",
    "US"             : "EBAY-US"
  },

  // URL templates
  _urls : {
    "roverTrack":      "http://rover.ebay.com/rover/1/#{ROVID}/4?mfe=#{AREA}&mpre=#{URL}",
    "autosuggest":     "http://anywhere.ebay.com/services/suggest/?q=#{QUERY}&s=#{SITE}&v=xml",
    //"autosuggest":     "http://anywhere.ebay.com/services/suggest/?q=#{QUERY}&s=#{SITE}",
    //"autosuggest":     "http://web.ebay.co.uk/autosuggest/output.php?q=#{QUERY}",
    //"autosuggest":     "http://include.ebaystatic.com/autofill/#{ALGORITHM}/#{SITEID}/#{VERSION}/#{QUERY}.js",
    "authAuth" :       "http://signin.#{DOMAIN}/ws/eBayISAPI.dll?SignIn&runame=#{RUNAME}&sessid=#{SESSID}&ruparams=#{RUPARAMS}",
    "authAuthSandbox": "http://signin.sandbox.#{DOMAIN}/ws/eBayISAPI.dll?SignIn&runame=#{RUNAME}&sessid=#{SESSID}&ruparams=#{RUPARAMS}",
    "anywhere":        "http://anywhere.#{DOMAIN}",
    "register":        "https://scgi.#{DOMAIN}/ws/eBayISAPI.dll?RegisterEnterInfo&UsingSSL=1&co_partnerId=2",
    "homePage":        "http://www.#{DOMAIN}",
    "buy":             "http://hub.#{DOMAIN}/buy",
    "sell":            "http://sell.#{DOMAIN}/sell",
    "myEbay":          "http://my.#{DOMAIN}/ws/eBayISAPI.dll?MyeBay",
    "myWorld":         "http://myworld.#{DOMAIN}/#{USERID}",
    "community":       "http://hub.#{DOMAIN}/community",
    "paypal":          "http://www.paypal.com",
    "appFeedback":     "http://forums.ebay.com/db1/topic/Ebay-Toolbar/Ebay-Sidebar-For/520122397",
    "myMessages":      "http://my.#{DOMAIN}/ws/eBayISAPI.dll?MyeBay&CurrentPage=MyeBayMyMessages",
    "search":          "http://shop.#{DOMAIN}/?_nkw=#{QUERY}",
    "listing":         "#{ITEMURL}",
    "listing2":        "http://cgi.#{DOMAIN}/ws/eBayISAPI.dll?ViewItem&item=#{ITEMID}",
    "catalog":         "http://catalog.#{DOMAIN}/?_pid=#{ITEMID}",
    "installTrack":    "http://rover.ebay.com/rover/1/#{TRACKTAG}/4?mfe=#{MFE}&adtype=0&mpt=#{UNIQUEID}",
    "bid":             "http://offer.#{DOMAIN}/ws/eBayISAPI.dll?MakeBid&item=#{ITEMID}",
    "bin":             "http://offer.#{DOMAIN}/ws/eBayISAPI.dll?BinConfirm&item=#{ITEMID}",
    "pay":             "http://payments.#{DOMAIN}/ws/eBayISAPI.dll?CheckoutProcessor&item=#{ITEMID}",
    "paid":            "http://payments.#{DOMAIN}/ws/eBayISAPI.dll?OrderAction&action=2&itemid=#{ITEMID}&pagetype=1883&ru=#{RU}",
    "sent":            "http://payments.#{DOMAIN}/ws/eBayISAPI.dll?OrderAction&action=4&itemid=#{ITEMID}&pagetype=1883&ru=#{RU}",
    "feedback":        "http://feedback.#{DOMAIN}/ws/eBayISAPI.dll?LeaveFeedback2&item=#{ITEMID}&useridto=#{USERIDTO}&useridfrom=#{USERIDFROM}",
    "relist":          "http://cgi.#{DOMAIN}/ws/eBayISAPI.dll?RelistItem&Item=#{ITEMID}",
    "find":            "http://search.#{DOMAIN}/search/search.dll?GetResult&sim=y&itemid=#{ITEMID}&query=#{QUERY}",
    "similar":         "http://cgi5.#{DOMAIN}/ws/eBayISAPI.dll?SellSimilarItem&itemId=#{ITEMID}",
    "userFeedback":    "http://feedback.#{DOMAIN}/ws/eBayISAPI.dll?ViewFeedback2&userid=#{USERID}",
    "revise":          "http://cgi5.#{DOMAIN}/ws/eBayISAPI.dll?ReviseListing&itemid=#{ITEMID}",
    "promote":         "http://cgi.#{DOMAIN}/ws/eBayISAPI.dll?PromoteItem&item=#{ITEMID}",
    "viewOther":       "http://cgi6.#{DOMAIN}/ws/eBayISAPI.dll?ViewSellersOtherItems&userid=#{USERID}",
    "askQuestion":     "http://contact.#{DOMAIN}/ws/eBayISAPI.dll?ShowCoreAskSellerQuestion&redirect=0&SSPageName=PageAskSellerQuestion_VI&requested=#{REQUESTED}&iid=#{IID}",
    "contactBuyer":    "http://contact.#{DOMAIN}/ws/eBayISAPI.dll?ReturnUserEmail&redirect=0&SSPageName=PageContactBuyer_VI&iid=#{IID}&requested=#{REQUESTED}",
    "orderDetails":    "http://payments.#{DOMAIN}/ws/eBayISAPI.dll?ViewPaymentStatus&ssPageName=STRK:MESOX:VPS&itemid=#{ITEMID}",
    "makeOffer":       "http://offer.#{DOMAIN}/ws/eBayISAPI.dll?MakeBestOffer&itemId=#{ITEMID}",
    "reviewOffers" :   "http://offer.#{DOMAIN}/ws/eBayISAPI.dll?ManageBestOffers&itemId=#{ITEMID}",
    "reviewOffer" :    "http://offer.#{DOMAIN}/ws/eBayISAPI.dll?ManageBestOffers&itemId=#{ITEMID}",
    "merchant":        "http://www.#{DOMAIN}/sch/merchant/#{USERID}",
    "savedSellers":    "http://my.#{DOMAIN}/ws/eBayISAPI.dll?MyEbayBeta&Column=Sellers&SavedSellers.Filter=&CurrentPage=MyeBaySavedSellers",
    "savedSearch":     "http://www.#{DOMAIN}/sch/#{SEARCHQUERY}",
    "savedSearches":   "http://my.#{DOMAIN}/ws/eBayISAPI.dll?MyEbay&CurrentPage=MyeBayAllFavorites",
    "viewDeals":       "http://#{DEALSDOMAIN}.#{DOMAIN}",
    "firstRun":        "http://anywhere.#{DOMAIN}/browser/firefox/welcome",
    "purchaseHistory": "http://offer.#{DOMAIN}/ws/eBayISAPI.dll?ViewBidsLogin&item=#{ITEMID}&guest=1",
    "support":         "https://getsatisfaction.com/ebayextensions/products/ebayextensions_ebay_extension_for_firefox",
    "help":            "https://getsatisfaction.com/ebayextensions",
    "TERMINATOR":      ""
  },

  popularCategoriesURLs : {
    US : {
      "motors": "http://www.ebay.com/motors",
      "fashion": "http://www.ebay.com/fashion",
      "homeAndGarden": "http://home.shop.ebay.com/",
      "sportingGoods": "http://www.ebay.com/vlp/sporting-goods"
    }
  },

  // Maps internal area names to rover area names expected by eBay
  // Names that map to "NOTWRAPPED" will not be rover-wrapped
  _roverNames : {
    "searchSuggestion": "sidebarSearch",
    "autosuggest":     "NOTWRAPPED",
    "connectButton":   "sidebarInfo",
    "itemButton":      "sidebarpri",
    "itemClick":       "sidebar",
    "itemDetails":     "itemDetails",
    "itemContext":     "sidebarContext",
    "postLogin":       "NOTWRAPPED",
    "searchBox":       "search",
    "sidebarButton":   "sidebarInfo",
    "statusbarButton": "menu",
    "toolbarButton":   "browserbutton",
    "installTracker":  "NOTWRAPPED",
    "desktopShortcut":   "desktop",
    "installerShortcut": "desktopInstaller",
    "emptyListText":   "sidebarnull",
    "forgotCredentials":   "forgotCredentials",
    "alertContents":   "alert",
    "alertButton":     "alertpri",
    "sidebarNotification": "message",
    "unwrappedLink":   "NOTWRAPPED",
    "favoriteSearchClick": "savedSearch",
    "dealClick":       "deal",
    "register":        "register",
    "firstRun":        "NOTWRAPPED",
    "platformNotification" : "campaign",
    "supportClick":        "support",
    "moreInformationClick": "moreInformationWebsite",
    "helpClick":       "help",
    "recentAlertClick":     "recentAlert",
    "recentBidClick":       "recentBid",
    "recentSearchClick":    "recentSearch",
    "recentViewClick":      "recentView",
    "recentWatchClick":     "recentWatch",
    "TERMINATOR":      ""
  },

  _shortcutParameterRead : false,

  priceRangeArray : [0, 5, 10, 15, 20, 25],

  conditionCode : {
    "NEW": 1000,
    "BRAND_NEW": 1000,
    "NEW_OTHER": 1500,
    "NEW_WITH_DEFECTS": 1750,
    "MANUFACTURER_REFURBISHED": 2000,
    "SELLER_REFURBISHED": 2500,
    "VERY_GOOD": 4000,
    "GOOD": 5000,
    "ACCEPTABLE": 6000,
    "FOR_PARTS": 7000,
    "REFURBISHED": 2000,
    "USED": 3000,
    "LIKE_NEW": 2750
  },

  /**
   * Initializes the resource.
   */
  init : function() {
    try {
      // Modules
      Cu.import("resource://ebaycompanion/common.js");
      Cu.import("resource://ebaycompanion/helpers/logger.js");
      Cu.import("resource://ebaycompanion/helpers/preferences.js");
      Cu.import("resource://ebaycompanion/helpers/stringBundle.js");

      this._rootPrefService =
        Cc["@mozilla.org/preferences-service;1"].
          getService(Ci.nsIPrefBranch2);

      this._prefBranch = new Preferences("extensions.ebaycomp.");
      this._stringBundle =
        new StringBundle("chrome://ebaycompanion/locale/strings.properties");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  /**
   * Checks whether the url is the auth & auth login page or not.
   * @param aURL the url to be checked.
   * @return the number indicates whether it is the login, accept url
   * or cancel url
   */
  isAuthAuthURL : function(aURL) {
    /* Regular expression used to match the login URL. */
    const AUTH_AUTH_LOGIN_URL_REGEXP =
      /^https:\/\/signin(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll\?SignIn/i;
    /* Regular expression used to match the intermediate URL when logging in. */
    const AUTH_AUTH_INTERMEDIATE_URL_REGEXP =
      /^https:\/\/scgi(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI.dll/i;
    /* Regular expression used to match the acceptance URL when logging in. */
    // XXX: on Sept. 23rd 2010 the accept/reject urls changed
    //const AUTH_AUTH_ACCEPT_URL_REGEXP =
    //  /^https:\/\/arribada(\.sandbox)?\.ebay(\.\w{2,3}){1,2}\/aw-secure\/auth_auth_thanks\.html\?(.*)/i;
    const AUTH_AUTH_ACCEPT_URL_REGEXP =
      /^https:\/\/signin(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll\?ThirdPartyAuthSucessFailure\&isAuthSuccessful=true/i;
      ///^https:\/\/arribada(\.sandbox)?\.ebay(\.\w{2,3}){1,2}\/aw-secure\/auth_auth_thanks\.html\?(.*)/i;
    /* Regular expression used to match the cancellation URL when logging in. */
    // XXX: on Sept. 23rd 2010 the accept/reject urls changed
    //const AUTH_AUTH_CANCEL_URL_REGEXP =
    //  /^https:\/\/arribada(\.sandbox)?\.ebay(\.\w{2,3}){1,2}\/aw-secure\/auth_auth_cancel\.html\?(.*)/i;
    const AUTH_AUTH_CANCEL_URL_REGEXP =
      /^https:\/\/signin(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll\?ThirdPartyAuthSucessFailure/i;
    /* Regular expression used to match the regular login URL. */
    const REGULAR_LOGIN_URL_REGEXP =
      /^https:\/\/signin(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll\?co_partnerid/i;
    const ITEM_LISTED_REGEXP =
     /^http:\/\/cgi\d{0,2}(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll/i;
    const ITEM_OFFER_PLACED_REGEXP =
      /^http:\/\/offer(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll\?ManageBestOffers\&itemid=(\d)+\&threadid=(\d)+\&OfferSubmittedPage=true/i;
    const ITEM_PURCHASED_REGEXP =
      /^http:\/\/offer(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}\/ws\/eBayISAPI\.dll/i;
    const EBAY_HOMEPAGE =
      /^http:\/\/www(\.sandbox)?(\.\w{2,4})?\.ebay(\.\w{2,3}){1,2}/i;
    try {
      var result = aURL.match(AUTH_AUTH_LOGIN_URL_REGEXP);
      if (result != null) {
        return 0;
      }
      result = aURL.match(AUTH_AUTH_INTERMEDIATE_URL_REGEXP);
      if (result != null) {
        return 1;
      }
      result = aURL.match(AUTH_AUTH_ACCEPT_URL_REGEXP);
      if (result != null) {
        return 2;
      }
      result = aURL.match(AUTH_AUTH_CANCEL_URL_REGEXP);
      if (result != null) {
        return 3;
      }
      result = aURL.match(REGULAR_LOGIN_URL_REGEXP);
      if (result != null) {
        return 4;
      }
      result = aURL.match(ITEM_LISTED_REGEXP);
      if (result != null) {
        return 5;
      }
      // item offer placed is more specific than item purchased REGEX.
      // because of that we have to test item offer placed first
      result = aURL.match(ITEM_OFFER_PLACED_REGEXP);
      if (result != null) {
        return 8;
      }
      result = aURL.match(ITEM_PURCHASED_REGEXP);
      if (result != null) {
        return 6;
      }
      result = aURL.match(EBAY_HOMEPAGE);
      if (result != null) {
        return 7;
      }
    } catch (e) {
      Logger.error(
        "An error occurred trying to check whether it is an auth & auth url.\n["
          + e.name + "] " + e.message);
    }

    return -1;
  },

  /**
   * Handles the state change on pages.
   * @param aWebProgress the web progress.
   * @param aRequest the request.
   * @param aStateFlags the state flags.
   * @param aStatus the status.
   */
  handleStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
    if ((aStateFlags & Ci.nsIWebProgressListener.STATE_STOP) && aRequest) {
      const ITEM_LISTED_URL = 5;

      // XXX: using a separate try as this QI throws exception with certain
      // frequency
      try {
        aRequest.QueryInterface(Ci.nsIChannel);
      } catch (e) { }

      try {
        if (aRequest.URI) {
          let url = aRequest.URI.spec;
          let resultPageNum = this.isAuthAuthURL(url);

          if (ITEM_LISTED_URL == resultPageNum) {
            if ((url.indexOf("fromMakeTrack=true") > -1 ||
                 url.indexOf("ViewItemMakeTrack") > -1) &&
                Datasource.activeAccount()) {
              // give 3 secs before triggering a hard update
              let callback = function() {
                let windowMediator =
                  Cc["@mozilla.org/appshell/window-mediator;1"].
                    getService(Ci.nsIWindowMediator);
                let mostRecentWindow =
                  windowMediator.getMostRecentWindow("navigator:browser");
                mostRecentWindow.EbayCompanion.ApiCoordinator.hardUpdate();
              };

              if (typeof(Timer) == "undefined") {
                Cu.import("resource://ebaycompanion/helpers/timer.js");
              }
              new Timer(callback, 3000, Timer.TYPE_ONE_SHOT);
            }
          }
        }
      } catch (e) {
        Logger.exception(e, Logger.DUMP_STACK)
      }
    }
  },

  /**
   * shortcutParameterRead getter
   * @return whether the desktop shortcut parameter has been read and used or
   * not
   */
  get shortcutParameterRead() {
    return this._shortcutParameterRead;
  },

  /**
   * shortcutParameterRead setter
   * @param aValue the value to be set
   */
  set shortcutParameterRead(aValue) {
    this._shortcutParameterRead = aValue;
  },

  /**
   * Getter
   */
  get prefBranch() {
    return this._prefBranch;
  },

  /**
   * root preferences service
   */
  get rootPrefService() {
    return this._rootPrefService;
  },

  /**
   * Getter
   */
  get stringBundle() {
    return this._stringBundle;
  },

  /**
   * Getter
   */
  get installTrackTags() {
    return this._installTrackTags;
  },

  /**
   * Getter
   */
  get localeSiteDataSelector() {
    return this._localeSiteDataSelector;
  },

  /**
   * Returns a url from our url template array
   * @param sourceAreaName used for rover tracking
   * @param templateName name of template
   * @param arguments for template placeholders
   */
  getUrl : function(sourceAreaName, templateName, args) {
    let templateString = this._urls[templateName];
    if (!templateString) {
      Logger.error("Bad URL Template name: '" + templateName + "'",
                     Logger.DUMP_STACK);
      return null;
    }

    // make the import until it is actually necessary
    if (typeof(Datasource) == "undefined") {
      Cu.import("resource://ebaycompanion/datasource.js");
    }

    // Get the URL
    let homeSite = Datasource.homeSite();

    // for first run page, if the homesite is not one of the supported ones, we
    // default to US
    if (templateName == "firstRun") {
      let firstRunDomains =["US",
                            "UK",
                            "Germany",
                            "Italy",
                            "France",
                            "Spain",
                            "Netherlands",
                            "Poland"];
      let setDefault = true;
      for (let i = 0; i < firstRunDomains.length; i++) {
        if (homeSite == firstRunDomains[i]) {
          setDefault = false;
          break;
        }
      }
      // we set it to null here so the next if sets it to the default value (US)
      if (setDefault) {
        homeSite = null;
      }
    }

    if (!homeSite) {
      homeSite = "US";
    }

    let template = new UrlTemplate(templateString);
    let domain = this._siteData[homeSite].dom;
    let activeAccount = Datasource.activeAccount();
    if (activeAccount && activeAccount.get("isSandboxAccount")) {
      domain = "sandbox." + domain;
    }
    template.setArg("DOMAIN", domain);
    if (args) {
      for each (let [arg, value] in Iterator(args)) {
        template.setArg(arg, value);
      }
    }

    // Perform Rover Wrapping if necessary
    let roverAreaName = this._roverNames[sourceAreaName];
    if (!roverAreaName) {
      Logger.error("Bad area name: '" + sourceAreaName + "'",
                   Logger.DUMP_STACK);
      return null;
    }

    let retUrl;
    if (roverAreaName == "NOTWRAPPED") {
      retUrl = template.url();
    } else {
      retUrl =
        this.roverUrl(template.url(), homeSite, sourceAreaName);
      Logger.log("Rovered url: " + retUrl);
    }

    return retUrl;
  },

  /**
   * Wraps the specified url with the rovering url
   * @param aUrl the url to be rovered
   * @param aHomeSite the home site to be used to rover the url
   * @param aSourceAreaName the source area name to be used to rover the url
   * @return the rovered url
   */
  roverUrl : function (aUrl, aHomeSite, aSourceAreaName) {
    let encodedUrl = encodeURIComponent(aUrl);
    let roverAreaName = this._roverNames[aSourceAreaName];

    let roverTemplate = new UrlTemplate(this._urls["roverTrack"]);
    let roverId = this._siteData[aHomeSite].rid;

    // if this is a San Disk build, we use the clientTrackingCode only if the
    // url being built is the one related to the desktop shortcut
    let isSanDiskBuild = this.prefBranch.get("SanDiskBuild");
    if ((aSourceAreaName.indexOf("desktopShortcut") != -1 ||
         aSourceAreaName.indexOf("installerShortcut") != -1 ) && isSanDiskBuild) {
      let sanDiskBuildRoverId =
        Datasource.getClientTrackingCode(aHomeSite, true);
      if (sanDiskBuildRoverId) {
        roverId = sanDiskBuildRoverId;
      }
    } else if (!isSanDiskBuild) {
      // if not, we continue the normal flow, where we try to use the
      // client-specific tracking codes
      // XXX: note that custom build is different than client-specific build!!!
      // check if there are client specific rover ids that we can use instead
      // of the default ones
      let clientRoverId = Datasource.getClientTrackingCode(aHomeSite, true);
      if (clientRoverId) {
        roverId = clientRoverId;
      }
    }

    roverTemplate.setArg("ROVID", roverId).
                  setArg("AREA", roverAreaName).
                  setArg("URL", encodedUrl);
    return roverTemplate.url();
  },

  /**
   * Returns an array of SiteId strings corresponding to supported eBay sites
   */
  supportedSites : function() {
    let sites = [];
    for each (let [site, data] in Iterator(this._siteData)) {
      sites.push(site);
    }
    return sites;
  },

  /**
   * Provides a numeric SiteId from a site name
   */
  siteIdForSite : function(siteString) {
    let siteData = this._siteData[siteString];
    if (!siteData) {
      // fall back to the US site
      siteData = this._siteData["US"];
    }
    return siteData.num;
  },

  /**
   * Provides a currency ID for the given string SiteId
   */
  currencyIdForSite : function(siteString) {
    let siteData = this._siteData[siteString];
    if (!siteData) {
      // fall back to the US siteId
      siteData = this._siteData["US"];
    }
    return siteData.cur;
  },

  /**
   * Provides an ISO-3166 country code for the given string SiteId
   */
  countryIso3166ForSite : function(siteString) {
    let siteData = this._siteData[siteString];
    if (!siteData) {
      // fall back to the US siteId
      siteData = this._siteData["US"];
    }
    return siteData.iso;
  },

  /**
   * Given a monetary value and its associated currency ID, return a string
   * that will make sense to the user.  For instance, a UK user might see "£10",
   * whereas a US user would see "GBP 10".
   */
  addCurrencySymbol : function(value, currency) {
    // make the import until it is actually necessary
    if (typeof(Datasource) == "undefined") {
      Cu.import("resource://ebaycompanion/datasource.js");
    }
    let homeSite = Datasource.homeSite();
    if (!homeSite) {
      homeSite = "US";
    }
    let siteData = this._siteData[homeSite];
    let nativeCurrency = this.currencyIdForSite(homeSite);

    let ret;
    if (currency == nativeCurrency) {
      // replace the placeholder with the given value
      ret = siteData.symb.replace("*", value);
    } else {
      ret = currency + " " + value;
    }

    return ret;
  },

  /**
   * Formats the number to add the thousand separators.
   * @param aValue the numerical value to be formatted.
   * @param aDecimals the fixed decimals value.
   * @return the formatted number.
   */
  formatNumber : function(aValue, aDecimals) {
    let decimalSeparator =
      this._stringBundle.getString("common.num.format.decimal").
        replace(/"/g, "");
    let thousandSeparator =
      this._stringBundle.getString("common.num.format.thousandSep").
        replace(/"/g, "");
    let value = aValue.toFixed(aDecimals);
    let valueString = String(value);
    let valueInteger = parseInt(value);
    let valueIntegerString = String(valueInteger);
    let valueIntegerStringSize = valueIntegerString.length;
    let valueDecimalIndex = valueString.indexOf(".") + 1;
    let valueDecimalString = "";
    let valueLabel = "";

    if (-1 != valueDecimalIndex) {
      valueDecimalString =
        valueString.substring(valueDecimalIndex, valueString.length);
    }
    for (let i = valueIntegerStringSize - 1; i >= 0; i--) {
      valueLabel = valueIntegerString[i] + valueLabel;
      if ((0 == (valueIntegerStringSize - i) % 3) && i > 0) {
        valueLabel = thousandSeparator + valueLabel;
      }
    }
    if (0 < aDecimals) {
      valueLabel += decimalSeparator + valueDecimalString;
    }

    return valueLabel;
  },

  /**
   * Converts a timestamp (in milliseconds) to a date string with the given
   * format.
   * @param aTimestamp a timestamp, in milliseconds.
   * @param aDateFormat a string that determines how the date will be formatted.
   * This is a RegExp replacement string with backreferences.
   * @return the date string in the given format.
   */
  formatDate : function(aTimestamp, aDateFormat) {
    let dateObject = new Date(Number(aTimestamp));
    let month = dateObject.getMonth() + 1;
    let dateString = "";

    dateString =
      aDateFormat.replace("$DD", this.formatTwoDigits(dateObject.getDate()));
    dateString = dateString.replace("$D", dateObject.getDate());

    if (0 <= dateString.indexOf("$ddd")) {
      let dayString =
        this._stringBundle.getString(
          "common.datetime.day." + (dateObject.getDay() + 1));

      dateString = dateString.replace("$ddd", dayString);
    }

    if (0 <= dateString.indexOf("$MMM")) {
      let monthString =
        this._stringBundle.getString("common.datetime.month." + month);

      dateString = dateString.replace("$MMM", monthString);
    }

    dateString = dateString.replace("$MM", this.formatTwoDigits(month));
    if (0 <= dateString.indexOf("$YYYY")) {
      dateString =
        dateString.replace(
          "$YYYY", String(dateObject.getFullYear()));
    } else {
      dateString =
        dateString.replace(
          "$YY", String(dateObject.getFullYear()).substring(2));
    }
    dateString =
      dateString.replace("$hh", this.formatTwoDigits(dateObject.getHours()));
    dateString =
      dateString.replace(
        "$mm", this.formatTwoDigits(dateObject.getMinutes()));
    dateString =
      dateString.replace(
        "$ss", this.formatTwoDigits(dateObject.getSeconds()));

    return dateString;
  },

  /**
   * Represent a non-negative number as a string of 2 digits. This is usually
   * needed to show hours, minutes and seconds on time displays.
   * @param aNumber the number to format to two digits. 0 <= aNumber < 100.
   * @return the formatted number. It's a string of 2 digits.
   */
  formatTwoDigits : function(aNumber) {
    let formattedNumber = "";

    if (aNumber >= 0 && aNumber < 100) {
      if (aNumber < 10) {
        formattedNumber += "0";
      }

      formattedNumber += aNumber;
    }

    return formattedNumber;
  },

  /**
   * Returns the correct Runame that should be used depending on whether we are
   * using the sandbox and which eBay site we are accessing.
   * @returns Runame string
   */
  getRuname : function(useSandbox, siteString) {
    let runame;
    if (!useSandbox) {
      // Production
      switch (siteString) {
        case "US":
          runame = "eBay_EU_MicroPr-eBayEUMi-98ba-4-dllhxv";
          break;
        default:
          runame = "eBay_EU_MicroPr-EBAYEUMICRI236G-hutwe";
          break;
      }
    } else {
      // Sandbox
      switch (siteString) {
        case "US":
          runame = "Glaxstar-GLAXSTARQ7HD16H-rtvtle";
          break;
        default:
          runame = "Glaxstar-GLAXSTARR2F4A7D-hgiiki";
          break;
      }
    }
    return runame;
  },

  /**
   * Builds a string with anchor elements to be used as notification content,
   * based on the string sent as parameter.
   * The source string is in the form "plain text [1 link] plain text...".
   * The number at the start of the link construct is set
   * as the order attribute to be used later to set link callbacks.
   * We can't set link callbacks here because that would require us to use a
   * string containing the callback code and that's hard to debug and looks
   * very bad. Instead, we leave that task to the notification binding.
   * This makes it possible to reorder the text during localisation without
   * changing the order of elements in the "callbacks" array.
   * The link labels are assigned the class "notificationLinkText" for styling.
   * @param aString the source string to be converted into notification contents
   */
  parseMarkupString : function(aString) {
    let content = "";

    try {
      let text = aString;
      let element;

      while (text.length > 0) {
        if (text.charAt(0) == "[") {
          // link
          element = "<a order='%1'>%2</a>";

          let endIndex = text.indexOf("]");
          let left = text.slice(0, endIndex + 1);
          left = left.replace("\n", "");
          let linkRe = /\[(\d+) (.*?)\]/g;
          linkRe.lastIndex = 0;
          let results = linkRe.exec(left);
          let index = results[1];

          element = element.replace("%1", results[1]).
                            replace("%2", results[2]);

          text = text.slice(endIndex + 1);
        } else {
          // text
          let endIndex = text.indexOf("[");
          if (endIndex == -1) {
            endIndex = text.length;
          }
          element = text.slice(0, endIndex);
          text = text.slice(endIndex);
        }
        content = content + element;
      }
    } catch(e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }

    return content;
  },

  /**
   * Returns the time left for an item as a string
   * @param aTimeLeft the time left to be converted
   */
  timeLeftAsString : function (aTimeLeft) {
    let acc = aTimeLeft / (1000 * 60 * 60 * 24);
    let days = Math.floor(acc);
    let leftover = acc - days;

    acc = leftover * 24;
    let hours = Math.floor(acc);
    leftover = acc - hours;

    acc = leftover * 60;
    let minutes = Math.floor(acc);
    leftover = acc - minutes;

    acc = leftover * 60;
    let seconds = Math.floor(acc);

    let vals = [days, hours, minutes, seconds];
    let units = ["day", "hour", "min", "sec"];

    // Determine which is the most significant non-zero unit
    let skip = 0;
    while (vals[skip] == 0) {
      skip++;
    }

    // Make sure we use at least one unit!
    skip = Math.min(vals.length-1, skip);

    // The following value determines how many units to display at a time
    let numValsToUse = 2;

    let params = [];
    let end = Math.min(skip + numValsToUse, vals.length);
    for (let i = skip; i < end; i++) {
      let propName = "item.timeleft." + units[i] + ".abbv";
      let text = this._stringBundle.getString(propName, [vals[i]]);
      params.push(text);
    }

    let whitespace = " ";
    let separator = whitespace.replace(/"/g, "");

    let timeLeftString = params.join(separator);

    return timeLeftString;
  },

  /**
   * Gets the UTF8 of the string, onnly necessary when not using string
   * bundles.
   * @param aString the string to convert.
   * @return the string converted.
   */
  getUTF8 : function(aString) {
    if (aString) { // if the string is not undefined, convert it
      let utf8;
      try {
        utf8 = decodeURIComponent(escape(aString));
      } catch (e) {
        utf8 = decodeURIComponent(encodeURIComponent(aString));
      }
      return utf8;
    } else {
      return null;
    }
  },

  /**
   * Given an ISO-8601 formatted time stamp, return a Date object
   * @param time
   * @returns Date object
   */
  dateFromIso8601 : function(time) {
    const dateRe =
      /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})\.(\d{3})Z$/;

    let results = dateRe.exec(time);
    if (!results) {
      Logger.error("Incompatible date format: " + time, Logger.DUMP_STACK);
      return Date();
    }

    // NOTE: The Date object deals with months from 0-11, not from 1-12
    let date = new Date();
    date.setUTCFullYear(results[1], results[2]-1, results[3]);
    date.setUTCHours(results[4], results[5], results[6], results[7]);

    return date;
  },

  /**
   * Compares two date objects.
   * @param aTimestampOne the unix timestamp to be compared.
   * @param aTimestampTwo the unix timestamp to be compared.
   * @return the object contains the time difference.
   */
  timeDifference : function(aTimestampOne, aTimestampTwo) {

    var startTime, endTime;
    if (aTimestampOne > aTimestampTwo) {
      startTime = aTimestampTwo;
      endTime = aTimestampOne;
    }
    else {
      startTime = aTimestampOne;
      endTime = aTimestampTwo;
    }

    var timeInSeconds = (endTime - startTime) / 1000;
    var difference = { };
    difference.seconds = parseInt((timeInSeconds) % 60);
    difference.minutes = parseInt((timeInSeconds / 60) % 60);
    difference.hours = parseInt((timeInSeconds / 3600) % 24);
    difference.days = parseInt((timeInSeconds / 86400));

    return difference;
  },

  /**
   * Indicates if an array contains a value
   * @param aArray the array to search in
   * @param aValue the value to search for
   * @returns true if the object was found, false otherwise
   */
  arrayContains : function(aArray, aValue) {
    for (var i = 0; i < aArray.length; i++) {
        if (aArray[i] === aValue) {
            return true;
        }
    }
    return false;
  },

  /**
   * Returns the SHA1 hash of the given string
   * @param aString the string to be hashed
   * @return the SHA1 hashed string.
   */
  sha1 : function(aString) {
    let converter =
      Cc["@mozilla.org/intl/scriptableunicodeconverter"].
        createInstance(Ci.nsIScriptableUnicodeConverter);

    // we use UTF-8 here, you can choose other encodings.
    converter.charset = "UTF-8";
    // result is an out parameter,
    // result.value will contain the array length
    let result = {};
    // data is an array of bytes
    let data = converter.convertToByteArray(aString, result);
    let ch =
      Cc["@mozilla.org/security/hash;1"].createInstance(Ci.nsICryptoHash);
    ch.init(ch.SHA1);
    ch.update(data, data.length);
    let hash = ch.finish(false);

    // return the two-digit hexadecimal code for a byte
    function toHexString(charCode)
    {
      return ("0" + charCode.toString(16)).slice(-2);
    }

    // convert the binary hash data to a hex string.
    return [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");
  },

  /**
   * Calculates and return the md5 checksum for the file passed as parameter
   * @param aFile the file to calculate its checksum
   */
  md5 : function(aFile) {
    let md5Checksum = "";

    let istream =
      Cc["@mozilla.org/network/file-input-stream;1"]
        .createInstance(Ci.nsIFileInputStream);
    // open for reading
    istream.init(aFile, 0x01, 0444, 0);
    let ch =
      Cc["@mozilla.org/security/hash;1"]
        .createInstance(Ci.nsICryptoHash);
    // we want to use the MD5 algorithm
    ch.init(ch.MD5);
    // this tells updateFromStream to read the entire file
    const PR_UINT32_MAX = 0xffffffff;
    ch.updateFromStream(istream, PR_UINT32_MAX);
    // pass false here to get binary data back
    let hash = ch.finish(false);

    // return the two-digit hexadecimal code for a byte
    function toHexString(charCode) {
      return ("0" + charCode.toString(16)).slice(-2);
    }

    // convert the binary hash data to a hex string.
    md5Checksum = [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");
    // s now contains your hash in hex

    return md5Checksum;
  },

  /**
   * Gets the global id.
   * @return the global id.
   */
  getGlobalId : function() {
    // make the import until it is actually necessary
    if (typeof(Datasource) == "undefined") {
      Cu.import("resource://ebaycompanion/datasource.js");
    }

    let site = Datasource.homeSite();
    let globalId;
    for (let siteValue in this._GLOBAL_ID_DATA) {
      if (siteValue == site) {
        globalId = this._GLOBAL_ID_DATA[siteValue];
        break;
      }
    }
    globalId = (null == globalId) ? "EBAY-US" : globalId;

    return globalId;
  }
};

/**
 * This object is used to manage URLs
 */
function UrlTemplate(url) {
  this._url = url;
}

UrlTemplate.prototype = {
  /**
   * Getter
   */
  url : function() {
    if (this.requiredArgs().length > 0) {
      Logger.warning("Returning UrlTemplate with unresolved placeholders:")
      Logger.warning(this._url, Logger.DUMP_STACK);
    }
    return this._url;
  },

  /**
   * Returns an array containing the names of the arguments that still need to
   * be passed
   */
  requiredArgs : function() {
    let argsRE = /#{(.*?)}/g;
    let result;
    let args = [];
    while (result = argsRE.exec(this._url)) {
      args.push(result[1]);
    }

    return args;
  },

  /**
   * Resolves a placeholder with an argument
   */
  setArg : function(arg, value) {
    if (typeof(value) == "undefined" || value == null) {
      Logger.warning("Undefined value passed for placeholder '" + arg + "'.",
                     Logger.DUMP_STACK);
      value = "";
    }
    let argRE = new RegExp("#{" + arg + "}", "gi");
    this._url = this._url.replace(argRE, value);
    return this;
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(Constants);
