/**
 * Copyright (c) 2008-2013 Appcoast. All rights reserved.
 */

var EXPORTED_SYMBOLS = ["Base64Encoder"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");

/**
 * Base64 encoder / decoder.
 */
Base64Encoder = {
  /* The hidden DOM window. Holds encoding and decoding functions. */
  _hiddenWindow : null,

  /**
   * Initializes the resource.
   */
  init : function() {
    this._hiddenWindow =
      Cc["@mozilla.org/appshell/appShellService;1"].
        getService(Ci.nsIAppShellService).hiddenDOMWindow;
  },

  /**
   * Base64-encode a binary string.
   * @param aBinaryString a binary string to encode.
   * @return string containing the base64 encoding.
   */
  encodeString : function(aBinaryString) {
    return this._hiddenWindow.btoa(aBinaryString);
  },

  /**
   * Base64-decode a string.
   * @param aString string to decode.
   * @return binary string representing the decoded value.
   * @throws Exception is the input string is invalid.
   */
  decodeString : function(aString) {
    return this._hiddenWindow.atob(aString);
  },

  /**
   * Helper function that turns a string into an array of numbers.
   * @param aString String to arrify.
   * @return Array holding numbers corresponding to the UCS character codes
   * of each character in aString.
   */
  arrayifyString : function(aString) {
    let output = new Array();

    for (let i = 0; i < aString.length; i++) {
      output.push(aString.charCodeAt(i));
    }

    return output;
  },

  /**
   * Helper function that turns an array of numbers into the string given by the
   * concatenation of the characters to which the numbers correspond (got
   * that?).
   * @param aArray Array of numbers representing characters.
   * @return Stringification of the array.
   */
  stringifyArray : function(aArray) {
    let output = new Array();

    for (let i = 0; i < aArray.length; i++) {
      output[i] = String.fromCharCode(aArray[i]);
    }

    return output.join("");
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(Base64Encoder);
