/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebRunner.
 *
 * The Initial Developer of the Original Code is Mozilla Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Mark Finkle <mark.finkle@gmail.com>, <mfinkle@mozilla.com>
 *   Cesar Oliveira <a.sacred.line@gmail.com>
 *   Matthew Gertner <matthew@allpeers.com>
 *   Juan Manuel Rodriguez <juan@glaxstar.com>
 *
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ["FileIOHelper"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const PR_WRONLY = 0x02;
const PR_CREATE_FILE = 0x08;
const PR_TRUNCATE = 0x20;

const PR_PERMS_FILE = 0644;

var FileIOHelper = {

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/helpers/logger.js");
  },

  // Returns the text content of a given nsIFile
  fileToString : function(file) {
    let data = "";
    try {
      // Get a nsIFileInputStream for the file
      let fis =
        Cc["@mozilla.org/network/file-input-stream;1"].
          createInstance(Ci.nsIFileInputStream);
      fis.init(file, -1, 0, 0);

      // Get an intl-aware nsIConverterInputStream for the file
      let is =
        Cc["@mozilla.org/intl/converter-input-stream;1"].
          createInstance(Ci.nsIConverterInputStream);
      is.init(
        fis,
        "UTF-8",
        1024,
        Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);

      // Read the file into string via buffer
      let buffer = {};
      while (is.readString(4096, buffer) != 0) {
        data += buffer.value;
      }

      // Clean up
      is.close();
      fis.close();


    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
    return data;
  },

  // Saves the given text string to the given nsIFile
  stringToFile : function(data, file) {
    try {
      // Get a nsIFileOutputStream for the file
      let fos =
        Cc["@mozilla.org/network/file-output-stream;1"].
          createInstance(Ci.nsIFileOutputStream);
      fos.init(
        file,
        PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE,
        (arguments.length == 3 ? arguments[2] : PR_PERMS_FILE),
        0);

      // Get an intl-aware nsIConverterOutputStream for the file
      let os =
        Cc["@mozilla.org/intl/converter-output-stream;1"].
          createInstance(Ci.nsIConverterOutputStream);
      os.init(fos, "UTF-8", 0, 0x0000);

      // Write data to the file
      os.writeString(data);

      // Clean up
      os.close();
      fos.close();
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(FileIOHelper);
