/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebRunner.
 *
 * The Initial Developer of the Original Code is Mozilla Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Mark Finkle <mark.finkle@gmail.com>, <mfinkle@mozilla.com>
 *   Cesar Oliveira <a.sacred.line@gmail.com>
 *   Matthew Gertner <matthew@allpeers.com>
 *   Juan Manuel Rodriguez <juan@glaxstar.com>
 *
 * ***** END LICENSE BLOCK ***** */

var EXPORTED_SYMBOLS = ["ShortcutHelper"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const PR_PERMS_DIRECTORY = 0755;
const MAC_ICNS_FILE = "ebay.icns";
const WIN_ICO_FILE = "ebay.ico";

var ShortcutHelper = {

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/common.js");
    Cu.import("resource://ebaycompanion/helpers/logger.js");
    Cu.import("resource://ebaycompanion/helpers/timer.js");
    Cu.import("resource://ebaycompanion/helpers/fileIOHelper.js");
    Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
    Cu.import("resource://ebaycompanion/constants.js");

    this._copyIcon();
  },

  /**
   * Copies the icon in the eBay folder.
   */
  _copyIcon : function() {
    let iconFile =
      Cc["@mozilla.org/file/directory_service;1"].
        getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
    iconFile.append("eBay Inc");
    iconFile.append(WIN_ICO_FILE);

    if (!iconFile.exists()) {
      let copied =
        UtilityHelper.extractFileFromZip(WIN_ICO_FILE, WIN_ICO_FILE);

      if (!copied) {
        let defaultFile = UtilityHelper.getDefaultsFolder(eBayComp.ExtensionId);

        defaultFile.append(WIN_ICO_FILE);
        defaultFile.copyTo(iconFile.parent, iconFile.leafName);
      }
    }
  },

  /**
   * Creates a shortcut in the desktop that opens firefox
   */
  createShortcut : function() {
    let dirSvc =
      Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties);

    // Locate the executable of firefox
    let target = dirSvc.get("XREExeF", Ci.nsIFile);
    // these lines were commented because if Firefox is not completely closed
    // (i.e. you leave the add-ons dialog opened), and you use the desktop
    // shortcut to open a new window, it will use de profile parameter as if
    // it were a url for the browser to open, which is NOT correct.
    /*let profService =
      Cc["@mozilla.org/toolkit/profile-service;1"].
        getService(Ci.nsIToolkitProfileService);
    let currentProfile = profService.selectedProfile;
    let profileName = currentProfile.name;*/
    let parameters = "-ebayComp";

    /*if (profileName != "default") {
      parameters += " -p " + profileName;
    }*/

    /* Check to see if were pointing to a binary (eg. firefox-bin).
     * We always want to point to firefox rather than firefox-bin,
     * because firefox will set up the library paths
     */
    if (target.leafName.search("-bin") != -1) {
      let target_shell = target.parent;
      target_shell.append(target.leafName.replace("-bin", ""));
      if (target_shell.exists()) {
        target = target_shell;
      }
    }

    let name =
      Constants.stringBundle.getString(
        "extensions." + eBayComp.ExtensionId + ".name");
    let desk = dirSvc.get("Desk", Ci.nsIFile);
    let appIcon =
      Cc["@mozilla.org/file/directory_service;1"].
        getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);

    appIcon.append("eBay Inc");

    switch (UtilityHelper.getOperatingSystem()) {
      /* Commented code for mac since we currently only have interest in
        windows
       case "MAC":
        appIcon.append(MAC_ICNS_FILE);
        this._createBundle(target, name, appIcon, desk, parameters);
        break;*/
      case "WINDOWS":
      case "VISTA":
      case "WIN7":
      case "WIN8":
        appIcon.append(WIN_ICO_FILE);
        // generate the shortcut creation script
        let shortcutScript =
          this._generateShortcutScript(
            target.path, name, appIcon.path, parameters);
        // now find the cscript executable
        let cscript =
          Cc["@mozilla.org/file/local;1"]
            .createInstance(Ci.nsILocalFile);
        cscript.initWithPath("C:\\Windows\\System32\\cscript.exe");

        // now run it!
        let process =
          Cc["@mozilla.org/process/util;1"]
            .createInstance(Ci.nsIProcess);
        process.init(cscript);

        // Run the process.
        // If first param is true, calling thread will be blocked until
        // called process terminates.
        // Second and third params are used to pass command-line arguments
        // to the process.
        let args = ["//B", shortcutScript.path];
        process.run(false, args, args.length);
        // then delete it, but give it a few seconds so we are sure it finished
        // creating the shortcut
        new Timer(function() {
            try {
              shortcutScript.remove(false);
            } catch(e) {
              Logger.exception(e, Logger.DUMP_STACK)
            }

          }, 20000, Timer.TYPE_ONE_SHOT);

        break;
      /* Commented code for linux since we currently only have interest in
        windows
      case "LINUX":
        this._createLinuxShortcut(target, name, desk, parameters);
        break;*/
      default:
        break;
    }
  },

  /**
   * Generates a temporal external javascript script to generate the desktop
   * shortcut
   */
  _generateShortcutScript : function(aTarget, aName, aIcon, aParameters) {

    let tempFile = eBayComp.getExtensionDirectory();
    tempFile.append("shortcutTempScript.js");

    // this is to replace the single backslashes in the path with double ones
    aTarget = aTarget.replace(/\\/g,"\\\\");
    aIcon = aIcon.replace(/\\/g,"\\\\");

    let data = "var WshShell = WScript.CreateObject(\"WScript.Shell\");\n" +
         "strDesktop = WshShell.SpecialFolders(\"Desktop\");\n" +
         "var oShellLink = WshShell.CreateShortcut(strDesktop + \"\\\\" + aName + ".lnk\");\n" +
         "oShellLink.TargetPath = \"" + aTarget + "\";\n" +
         "oShellLink.WindowStyle = 1;\n" +
         "oShellLink.IconLocation = \""+ aIcon + ", 0\";\n" +
         "oShellLink.Description = \"" + aName +"\";\n" +
         "oShellLink.WorkingDirectory = strDesktop;\n" +
         "oShellLink.Arguments = \"" + aParameters +"\";\n" +
         "oShellLink.Save();";

    FileIOHelper.stringToFile(data, tempFile, 0755);

    return tempFile;
  },

  /**
   * Creates the shortcut bundle that opens firefox on mac OS X
   * @param aTarget the target application to be run
   * @param aName the shortcut name
   * @param aIcon the icon to be used
   * @param aLocation the directory where the shortcut will be created
   * @param aParameters a string with the parameters to be added to the shortcut
   * target
   * @return the created bundle
   */
  _createBundle : function(aTarget, aName, aIcon, aLocation, aParameters) {
    let contents =
    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
    "<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n" +
    "<plist version=\"1.0\">\n" +
    "<dict>\n" +
    "<key>CFBundleExecutable</key>\n" +
    "<string>" + aName + "</string>\n" +
    "<key>CFBundleIconFile</key>\n" +
    "<string>" + aIcon.leafName + "</string>\n" +
    "</dict>\n" +
    "</plist>";

    aLocation.append(aName + ".app");
    if (aLocation.exists())
      aLocation.remove(true);
    aLocation.create(Ci.nsIFile.DIRECTORY_TYPE, PR_PERMS_DIRECTORY);

    let bundle = aLocation.clone();

    aLocation.append("Contents");
    aLocation.create(Ci.nsIFile.DIRECTORY_TYPE, PR_PERMS_DIRECTORY);

    let info = aLocation.clone();
    info.append("Info.plist");
    FileIOHelper.stringToFile(contents, info);

    let resources = aLocation.clone();
    resources.append("Resources");
    resources.create(Ci.nsIFile.DIRECTORY_TYPE, PR_PERMS_DIRECTORY);
    aIcon.copyTo(resources, aIcon.leafName);

    let macos = aLocation.clone();
    macos.append("MacOS");
    macos.create(Ci.nsIFile.DIRECTORY_TYPE, PR_PERMS_DIRECTORY);

    let cmd = "#!/bin/sh\nexec \"" + aTarget.path + "\" " + aParameters;
    let script = macos.clone();
    script.append(aName);
    FileIOHelper.stringToFile(cmd, script, 0755);

    return bundle;
  },

  /**
   * Creates the shortcut bundle that opens firefox on Linux
   * @param aTarget the target application to be run
   * @param aName the shortcut name
   * @param aLocation the directory where the shortcut will be created
   * @param aParameters a string with the parameters to be added to the shortcut
   * target
   */
  _createLinuxShortcut : function (aTarget, aName, aLocation, aParameters) {
    let cmd = "#!/bin/sh\nexec \"" + aTarget.path + "\" " + aParameters;
    let script = aLocation.clone();
    script.append(aName);
    FileIOHelper.stringToFile(cmd, script, 0755);
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(ShortcutHelper);
