/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["Item"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/utilityHelper.js");
Cu.import("resource://ebaycompanion/helpers/objectHelper.js");
Cu.import("resource://ebaycompanion/constants.js");

/**
 * Ebay item object.
 * @param itemId
 */
function Item(itemId) {
  if (itemId) {
    this.set("itemId", itemId);
  }
  this._transaction = null;
  this.set("userQuantityWinning", 0);
  this.set("thumbnailUrl", "");
  this.set("pageViews", 0);
  this.set("convertedMaxBid", 0);
  this.set("convertedMaxBidCurrency", '');
  this.set("bestOfferEnabled", false);
  this.set("bestOffer", 0);
  this.set("convertedBestOffer", 0);
  this.set("bestOfferCount", 0);
  this.set("userMaxBid", 0);
  this.set("userQuantityBidFor", 0);
  this.set("convertedCurrentPrice", 0);
  this.set("convertedCurrentPriceCurrency", '');
  this.set("numWatching", 0);
  this.set("shippingTermsInDescription", false);

}

/* Item state constants. */
Item.ITEM_STATE_WATCHING = 0;
Item.ITEM_STATE_BUYING_ITEM_WON = 1;
Item.ITEM_STATE_BUYING_ITEM_LOST = 2;
Item.ITEM_STATE_BUYING_ITEM_LOST_RESERVE_NOT_MET = 3;
Item.ITEM_STATE_BUYING_SUCCESS = 4;
Item.ITEM_STATE_BUYING_OUTBID = 5;
Item.ITEM_STATE_BUYING_RESERVE_NOT_MET = 6;
Item.ITEM_STATE_SELLING = 7;
Item.ITEM_STATE_SELLING_SUCCESS = 8;
Item.ITEM_STATE_SELLING_RESERVE_NOT_MET = 9;
Item.ITEM_STATE_SELLING_ITEM_SOLD = 10;
Item.ITEM_STATE_SELLING_ITEM_UNSOLD = 11;
Item.ITEM_STATE_SELLING_ITEM_UNSOLD_RESERVE_NOT_MET = 12;
// classified ads
Item.ITEM_STATE_WATCHING_CLASSIFIED_AD = 13;
Item.ITEM_STATE_SELLING_CLASSIFIED_AD = 14;
// best offer
Item.ITEM_STATE_BEST_OFFER_PENDING = 15;
Item.ITEM_STATE_BEST_OFFER_DECLINED = 16;
Item.ITEM_STATE_BEST_OFFER_EXPIRED = 17;
Item.ITEM_STATE_BEST_OFFER_COUNTERED = 18;
Item.ITEM_STATE_SELLING_WITH_OFFERS = 19;
Item.ITEM_STATE_BEST_OFFER_ITEM_SOLD = 20;
Item.ITEM_STATE_BEST_OFFER_ITEM_WON = 21;
Item.ITEM_STATE_WATCHING_BEST_OFFER = 22;
Item.ITEM_STATE_SELLING_BEST_OFFER = 23;

/* Item Type constants */
Item.ITEM_TYPE_WATCHING = "Watching";
Item.ITEM_TYPE_WON = "Won";
Item.ITEM_TYPE_LOST = "Lost";
Item.ITEM_TYPE_SOLD = "Sold";
Item.ITEM_TYPE_UNSOLD = "Unsold";
Item.ITEM_TYPE_BIDDING = "Bidding";
Item.ITEM_TYPE_SELLING = "Selling";
Item.ITEM_TYPE_BEST_OFFER = "Best Offer";

Item.prototype = {
  /**
   * Used to determine the type of the object
   */
  constructor : Item,

  /**
   * Types for each property of this object
   */
  propertyTypes : {
    itemId :               "number",
    title :                "string",
    sellerUserId :         "string",
    isEnded :              "boolean",
    listingFormat :        "listingFormat",
    currentPrice :         "number",
    currency :             "string",
    convertedCurrentPrice : "number",
    convertedCurrentPriceCurrency : "string",
    convertedMaxBid :        "number",
    convertedMaxBidCurrency : "string",
    shippingCost :         "string",
    shippingTermsInDescription : "boolean",
    shipToLocations :      "string",
    shippingType :         "string",
    quantitySold :         "number",
    quantityRemaining :    "number",
    quantity :             "number",
    startTime :            "string",
    endTime :              "string",
    relistedItemId :       "number",
    imageUrl :             "string",
    thumbnailUrl :         "string",
    pageViews :            "number",
    hitCounterType :       "string",
    hasBuyItNow :          "boolean",
    buyItNowPrice :        "number",
    numBids :              "number",
    isReserveMet :         "boolean",
    userMaxBid :           "number",
    highBidderId :         "string",
    highBidderFeedbackScore : "number",
    userQuantityBidFor :   "number",
    userQuantityWinning :  "number",
    sellerFeedbackRating : "number",
    sellerFeedbackPercent : "number",
    numWatching :          "number",
    type :                 "string",
    bestOfferEnabled :     "boolean",
    bestOffer :            "number",
    bestOfferCurrency :    "string",
    bestOfferStatus :      "string",
    convertedBestOffer :   "number",
    convertedBestOfferCurrency : "string",
    bestOfferCount :       "number",
    viewItemUrl :          "string",
    condition :            "number",
    conditionDisplayName : "string",
    returnsInfo :          "string",
    paymentInfo :          "string"
  },

  /**
   * XML nodes related to each property
   */
  propertyXML : {
    itemId :               "ItemID",
    title :                "Title",
    sellerUserId :         "Seller.UserID",
    isEnded :              "",
    listingFormat :        "ListingType",
    currentPrice :         "SellingStatus.CurrentPrice|CurrentPrice",
    currency :             "SellingStatus.CurrentPrice.@currencyID|CurrentPrice.@currencyID",
    convertedCurrentPrice : "SellingStatus.ConvertedCurrentPrice|ConvertedCurrentPrice",
    convertedCurrentPriceCurrency : "SellingStatus.ConvertedCurrentPrice.@currencyID|ConvertedCurrentPrice.@currencyID",
    convertedMaxBid :        "ConvertedMaxBid",
    convertedMaxBidCurrency : "ConvertedMaxBid.@currencyID",
    shippingCost :         "",
    shippingTermsInDescription : "ShippingTermsInDescription",
    shipToLocations :      "ShipToLocations",
    shippingType :         "ShippingType|ShippingCostSummary.ShippingType",
    quantitySold :         "SellingStatus.QuantitySold|QuantitySold",
    quantityRemaining :    "",
    quantity :             "Quantity",
    startTime :            "ListingDetails.StartTime|StartTime",
    endTime :              "ListingDetails.EndTime|EndTime",
    relistedItemId :       "RelistedItemID",
    imageUrl :             "PictureDetails.PictureURL|PictureURL",
    thumbnailUrl :         "PictureDetails.GalleryURL|GalleryURL",
    pageViews :            "HitCount",
    hitCounterType :       "HitCounter",
    hasBuyItNow :          "ListingDetails.BuyItNowAvailable",
    buyItNowPrice :        "BuyItNowPrice|CurrentPrice",
    numBids :              "SellingStatus.BidCount|BidCount",
    isReserveMet :         "SellingStatus.ReserveMet|ReserveMet",
    userMaxBid :           "BiddingDetails.MaxBid|MaxBid",
    highBidderId :         "SellingStatus.HighBidder.UserID|HigherBidder.UserID",
    highBidderFeedbackScore : "SellingStatus.HighBidder.FeedbackScore|HighBidder.FeedbackScore",
    userQuantityBidFor :   "BiddingDetails.QuantityBid|QuantityBid",
    userQuantityWinning :  "BiddingDetails.QuantityWon|QuantityWon",
    sellerFeedbackRating : "Seller.FeedbackScore",
    sellerFeedbackPercent : "Seller.PositiveFeedbackPercent",
    numWatching :          "WatchCount",
    type :                 "",
    bestOfferEnabled :     "BestOfferEnabled",
    bestOffer :            "BestOfferDetails.BestOffer",
    bestOfferCurrency :    "BestOfferDetails.BestOffer.@currencyID",
    bestOfferStatus :      "BestOfferDetails.BestOfferStatus",
    convertedBestOffer :   "BestOfferDetails.ConvertedBestOffer",
    convertedBestOfferCurrency : "BestOfferDetails.ConvertedBestOffer.@currencyID",
    bestOfferCount :       "",
    viewItemUrl :          "ListingDetails.ViewItemURLForNaturalSearch|ViewItemURLForNaturalSearch",
    condition :            "",
    conditionDisplayName : "",
    returnsInfo :          "",
    paymentInfo :          ""
  },

  get type() {
    return this.get("type");
  },

  set type(aValue) {
    this.set("type", aValue);
  },

  /**
   * Returns a copy of this object.
   * @return the copy of this object.
   */
  copy : function() {
    let copy = new Item(this.get("itemId"));
    copy.updateQuietlyTo(this);
    if (this.type) {
      copy.type = this.type;
    }
    return copy;
  },

  /**
   * Updates this object with the new object.
   * @param aNewObject the new object.
   * @param aFlags the set of flags.
   * @param aIgnoreFields an array with the names of the fields to be ignored
   * during the update
   * @return the result.
   */
  updateTo : function(aNewObject, aFlags, aIgnorePropertiesArray) {
    return ObjectHelper.updateObject(
      this, aNewObject, Topics.ITEM_PROPERTY_UPDATED, aFlags,
      aIgnorePropertiesArray);
  },

  /**
   * Updates quietly this object with the new object.
   * @param aNewObject the new object.
   * @return the result.
   */
  updateQuietlyTo : function(aNewObject) {
    return this.updateTo(aNewObject, ObjectHelper.flags.NO_NOTIFICATIONS);
  },

  /**
   * Updates a property of the object.
   * @param aProperty the property name.
   * @param aValue the property value.
   * @return the result.
   */
  updateProperty : function(aProperty, aValue) {
    return ObjectHelper.updateProperty(
      this, aProperty, aValue, Topics.ITEM_PROPERTY_UPDATED);
  },

  /**
   * Returns the value of the given property.
   * @param aProperty the property name.
   * @return the property value.
   */
  get : function(aProperty) {
    return ObjectHelper.getProperty(this, aProperty);
  },

  /**
   * Sets the given property to the given value, checking that the value is of
   * the correct type for the property
   * @param aProperty the property name.
   * @param aValue the property value.
   */
  set : function(aProperty, aValue) {
    aValue = ObjectHelper.filterValue(
      aProperty, this.propertyTypes[aProperty], aValue);
    this["_" + aProperty] = aValue;
  },

  /**
   * Returns the transaction related to this item or null if the item doesn't
   * have any.
   * @return the transaction.
   */
  get transaction() {
    return this._transaction;
  },

  /**
   * Sets this item transaction to the given value.
   * @param aValue the value to be set.
   */
  set transaction(aValue) {
    this._transaction = aValue;
  },

  /**
   * Gets the item currenct state.
   * @param aUserId the current active user id.
   * @param aEbayTime the current ebay time.
   * @param aOriginalQuantitySold the original quantity sold value.
   * @return the item current state.
   */
  getCurrentState : function(aUserId) {

    let state = null;
    let EC_LISTING_TYPE_CLASSIFIED = "AdType";
    let EC_LISTING_TYPE_LEAD_GENERATION = "LeadGeneration";
    let EC_LISTING_TYPE_FIXED = "FixedPriceItem";
    let EC_LISTING_TYPE_STORE_FIXED = "StoresFixedPrice";

    if (this.type == Item.ITEM_TYPE_WATCHING) {
      // watching item.
      if (EC_LISTING_TYPE_LEAD_GENERATION == this.get("listingFormat") ||
          EC_LISTING_TYPE_CLASSIFIED == this.get("listingFormat")) {
        state = Item.ITEM_STATE_WATCHING_CLASSIFIED_AD;
      } else if (this.get("bestOfferEnabled")) {
        state = Item.ITEM_STATE_WATCHING_BEST_OFFER;
      } else {
        state = Item.ITEM_STATE_WATCHING;
      }
    } else if (this.type == Item.ITEM_TYPE_WON) {
      state = Item.ITEM_STATE_BUYING_ITEM_WON;
      // depending on the final transaction prices, this could be a regular
      // transaction or a best offer one if the transaction price is
      // different to the item's current price.
      if (this.transaction &&
          this.transaction.get("transactionPrice") > 0 &&
          this.get("bestOfferEnabled") &&
          this.get("currentPrice") >
          this.transaction.get("transactionPrice")) {
        state = Item.ITEM_STATE_BEST_OFFER_ITEM_WON;
      }
    } else if (this.type == Item.ITEM_TYPE_LOST) {
      if (this.get("isReserveMet")) {
        state = Item.ITEM_STATE_BUYING_ITEM_LOST;
      } else {
        state = Item.ITEM_STATE_BUYING_ITEM_LOST_RESERVE_NOT_MET;
      }
    } else if (this.type == Item.ITEM_TYPE_SOLD) {
      state = Item.ITEM_STATE_SELLING_ITEM_SOLD;
      // depending on the final transaction prices, this could be a regular
      // transaction or a best offer one if the transaction price is
      // different to the item's current price.
      if (this.transaction &&
          this.transaction.get("transactionPrice") > 0 &&
          this.get("bestOfferEnabled") &&
          this.get("currentPrice") >
          this.transaction.get("transactionPrice")) {
        state = Item.ITEM_STATE_BEST_OFFER_ITEM_SOLD;
      }
    } else if (this.type == Item.ITEM_TYPE_UNSOLD) {
      if (this.get("isReserveMet")) {
        state = Item.ITEM_STATE_SELLING_ITEM_UNSOLD;
      } else {
        state = Item.ITEM_STATE_SELLING_ITEM_UNSOLD_RESERVE_NOT_MET;
      }
    } else if (this.type == Item.ITEM_TYPE_BIDDING) {
      if (this.get("userQuantityWinning") > 0 &&
          this.get("highBidderId").toLowerCase() == aUserId.toLowerCase()) {
        state = Item.ITEM_STATE_BUYING_SUCCESS;
      } else if (this.get("userMaxBid") > 0 && !this.get("isReserveMet")) {
        state = Item.ITEM_STATE_BUYING_RESERVE_NOT_MET;
      } else {
        state = Item.ITEM_STATE_BUYING_OUTBID;
      }
    } else if (this.type == Item.ITEM_TYPE_SELLING) {
      if (this.get("numBids") > 0 && !this.get("isReserveMet")) {
        state = Item.ITEM_STATE_SELLING_RESERVE_NOT_MET;
      } else if (this.get("numBids") > 0) {
        state = Item.ITEM_STATE_SELLING_SUCCESS;
      } else {
        state = Item.ITEM_STATE_SELLING;
        if (this.get("bestOfferCount") > 0) {
          state = Item.ITEM_STATE_SELLING_WITH_OFFERS;
        } else if (this.get("bestOfferEnabled")) {
        state = Item.ITEM_STATE_SELLING_BEST_OFFER;
        }
      }
    } else if (this.type == Item.ITEM_TYPE_BEST_OFFER) {
      if (this.get("bestOfferStatus").toLowerCase() == "pending") {
        state = Item.ITEM_STATE_BEST_OFFER_PENDING;
      } else if (this.get("bestOfferStatus").toLowerCase() == "declined") {
        state = Item.ITEM_STATE_BEST_OFFER_DECLINED;
      } else if (this.get("bestOfferStatus").toLowerCase() == "expired") {
        state = Item.ITEM_STATE_BEST_OFFER_EXPIRED;
      } else if (this.get("bestOfferStatus").toLowerCase() == "countered") {
        state = Item.ITEM_STATE_BEST_OFFER_COUNTERED;
      } else {
        // not pending or declined? the state is watching then
        state = Item.ITEM_STATE_WATCHING;
      }
    }

    return state;
  },

  fromJXONObject : function(aJXONObject) {
    var item = ObjectHelper.fromJXON(aJXONObject, this);
    this.updateTo(item);

    // set the best offer count (if any)

    let bestOfferCount = (aJXONObject["BestOfferDetails"] ? aJXONObject["BestOfferDetails"]["BestOfferCount"] : 0);
    if (bestOfferCount) {
      this.set("bestOfferCount", bestOfferCount);
    }

    // set a default value for reserve met field, in case it is not returned
    // by the api calls
    this.set("isReserveMet", !(aJXONObject["ReserveMet"] == false));

    // isEnded
    this.set("isEnded", (aJXONObject["TimeLeft"] == "PT0S"));


    // quantityRemaining
    let quantityRemaining = this.get("quantity") - this.get("quantitySold")
    quantityRemaining = isNaN(quantityRemaining) ? 0 : quantityRemaining;
    this.set("quantityRemaining", quantityRemaining);

    // startTime and endTime
    var startDate;
    if (aJXONObject["ListingDetails"] && aJXONObject["ListingDetails"]["StartTime"]) {
      startDate = aJXONObject["ListingDetails"]["StartTime"];
    } else {
      startDate = aJXONObject["StartTime"];
    }

    if (startDate)
      this.set("startTime", String(startDate.getTime()));

    var endDate;
    if (aJXONObject["ListingDetails"] && aJXONObject["ListingDetails"]["EndTime"]) {
      endDate = aJXONObject["ListingDetails"]["EndTime"];
    } else {
      endDate = aJXONObject["EndTime"];
    }
    if (endDate)
      this.set("endTime", String(endDate.getTime()));

    // hasBuyItNow and buyItNowPrice
    var hasBuyItNow;
    var buyItNowPrice =
      this.get("buyItNowPrice") ? this.get("buyItNowPrice") : 0;

    switch (this.get("listingFormat")) {
      case "Chinese":
      case "Dutch":
        if (buyItNowPrice == 0) {
          hasBuyItNow = false;
        } else {
          hasBuyItNow = this.get("hasBuyItNow") ? this.get("hasBuyItNow") : false;
        }
        break;
      default:
        hasBuyItNow = true;
        buyItNowPrice = this.get("currentPrice") ? this.get("currentPrice") : 0;
    }
    this.set("hasBuyItNow", hasBuyItNow);
    this.set("buyItNowPrice", buyItNowPrice);

    // the shipping cost info
    let shippingCost = aJXONObject["ShippingCostSummary"];
    if (shippingCost != null) {
      this.set("shippingCost", JSON.stringify(shippingCost));
    }

    // the condition info
    let condition = aJXONObject["ConditionID"];
    if (condition != null) {
      this.set("condition", condition);
    } else {
      this.set("condition", 0);
    }

    // the condition info
    let conditionDisplayName = aJXONObject["ConditionDisplayName"];
    if (conditionDisplayName != null && conditionDisplayName != undefined) {
      this.set("conditionDisplayName", conditionDisplayName);
    }

    // the returns info
    let returnsInfo = aJXONObject["ReturnPolicy"];
    if (returnsInfo != null) {
      this.set("returnsInfo", JSON.stringify(returnsInfo));
    }

    // the payments info
    let paymentInfo = aJXONObject["PaymentMethods"];
    if (paymentInfo != null) {
      this.set("paymentInfo", JSON.stringify(paymentInfo));
    }

  }
};
