/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["RecentWatch"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/objectHelper.js");

/**
 * eBay Recent Watch details
 * @param userId the id of the user
 * @param item id the item id
 */
function RecentWatch(userId, itemId) {
  this.set("userId", userId);
  this.set("itemId", itemId);
}

RecentWatch.prototype = {
  /**
   * Used to determine the type of the object
   */
  constructor : RecentWatch,

  /**
   * Types for each property of this object
   */
  propertyTypes : {
    userId :            "string",
    itemId :            "number",
    timestamp :         "string"
  },

  /**
   * Returns a copy of this object
   */
  copy : function() {
    let copy = new RecentWatch(
                 this.get("userId"),
                 this.get("itemId"));
    copy.updateQuietlyTo(this);
    return copy;
  },

  /**
   * updateTo
   */
  updateTo : function(newObject, flags) {
    return ObjectHelper.
             updateObject(
               this, newObject, "ebay-recent-watch-property-updated", flags);
  },

  /**
   * updateQuietlyTo
   */
  updateQuietlyTo : function(newObject) {
    return this.updateTo(newObject, ObjectHelper.flags.NO_NOTIFICATIONS);
  },

  /**
   * updateProperty
   */
  updateProperty : function(property, value) {
    return ObjectHelper.
             updateProperty(this, property, value,
                            "ebay-recent-watch-property-updated");
  },

  /**
   * Returns the value of the given property
   */
  get : function(property) {
    return ObjectHelper.getProperty(this, property);
  },

  /**
   * Sets the given property to the given value, checking that the value is of
   * the correct type for the property
   */
  set : function(property, value) {
    value = ObjectHelper.
              filterValue(property, this.propertyTypes[property], value);
    this["_" + property] = value;
  }
};
