/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["SystemMessage"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/objectHelper.js");

/**
 * eBay System Message
 * @param aMessageId eBay message ID
 * @param aUserId the id of the user who got this message
 */
function SystemMessage(aMessageId, aUserId) {
  this.set("messageId", aMessageId);
  this.set("userId", aUserId);
  this.set("numberOfViews", 1);
  this.set("maxNumberOfViews", 1);
}

SystemMessage.prototype = {
  /**
   * Used to determine the type of the object
   */
  constructor : SystemMessage,

  /**
   * Types for each property of this object
   */
  propertyTypes : {
    messageId :         "number",
    userId :            "string",
    jsonBody :          "string",
    type :              "string",
    numberOfViews :     "number",
    maxNumberOfViews :  "number",
    validFromTime :     "number",
    validToTime :       "number"
  },

  /**
   * Returns a copy of this object
   */
  copy : function() {
    let copy = new SystemMessage(this.get("messageId"), this.get("userId"));
    copy.updateQuietlyTo(this);
    return copy;
  },

  /**
   * updateTo
   */
  updateTo : function(newObject, flags) {
    return ObjectHelper.updateObject(
      this, newObject, Topics.SYSTEM_MESSAGE_PROPERTY_UPDATED, flags);
  },

  /**
   * updateQuietlyTo
   */
  updateQuietlyTo : function(newObject) {
    return this.updateTo(newObject, ObjectHelper.flags.NO_NOTIFICATIONS);
  },

  /**
   * updateProperty
   */
  updateProperty : function(property, value) {
    return ObjectHelper.updateProperty(
      this, property, value, Topics.SYSTEM_MESSAGE_PROPERTY_UPDATED);
  },

  /**
   * Returns the value of the given property
   */
  get : function(property) {
    return ObjectHelper.getProperty(this, property);
  },

  /**
   * Sets the given property to the given value, checking that the value is of
   * the correct type for the property
   */
  set : function(property, value) {
    value = ObjectHelper.
              filterValue(property, this.propertyTypes[property], value);
    this["_" + property] = value;
  }
};
