/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["TrackingCode"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/helpers/objectHelper.js");

/**
 * eBay client specific TrackingCode records
 * @param country the country for which use the TrackingCode code
 * @param isRover whether this is a rover id or a installation TrackingCode code
 * @param value the TrackingCode code value
 */
function TrackingCode(country, isRover, value) {
  this.set("country", country);
  this.set("isRover", isRover);
  this.set("value", value);
}

TrackingCode.prototype = {
  /**
   * Used to determine the type of the object
   */
  constructor : TrackingCode,

  /**
   * Types for each property of this object
   */
  propertyTypes : {
    country :            "string",
    isRover :            "boolean",
    value   :            "string"
  },

  /**
   * Returns a copy of this object
   */
  copy : function() {
    let copy = new TrackingCode(
                 this.get("country"),
                 this.get("isRover"),
                 this.get("value"));
    copy.updateQuietlyTo(this);
    return copy;
  },

  /**
   * updateTo
   */
  updateTo : function(newObject, flags) {
    return ObjectHelper.
             updateObject(
               this, newObject, "ebay-tracking-code-property-updated", flags);
  },

  /**
   * updateQuietlyTo
   */
  updateQuietlyTo : function(newObject) {
    return this.updateTo(newObject, ObjectHelper.flags.NO_NOTIFICATIONS);
  },

  /**
   * updateProperty
   */
  updateProperty : function(property, value) {
    return ObjectHelper.
             updateProperty(this, property, value,
                            "ebay-tracking-code-property-updated");
  },

  /**
   * Returns the value of the given property
   */
  get : function(property) {
    return ObjectHelper.getProperty(this, property);
  },

  /**
   * Sets the given property to the given value, checking that the value is of
   * the correct type for the property
   */
  set : function(property, value) {
    value = ObjectHelper.
              filterValue(property, this.propertyTypes[property], value);
    this["_" + property] = value;
  }
};
