/**
 * Copyright (C) 2013 eBay Inc. All Rights Reserved.
 */
var EXPORTED_SYMBOLS = ["ItemService"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://ebaycompanion/common.js");
Cu.import("resource://ebaycompanion/helpers/logger.js");
Cu.import("resource://ebaycompanion/helpers/observers.js");
Cu.import("resource://ebaycompanion/helpers/timer.js");
Cu.import("resource://ebaycompanion/constants.js");
Cu.import("resource://ebaycompanion/datasource.js");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://ebaycompanion/objects/alert.js");
Cu.import("resource://ebaycompanion/objects/item.js");

/**
 * Recent Item Service.
 */
var ItemService = {

  /**
   * Initializes the service.
   */
  init: function() {
  },

  /**
   * Calls to show the item details.
   * @param aItemNode the itemNode to see its details, null to hide the
   * details panel.
   * @param aSkipAnimation true if skips animation.
   */
  showItemDetails : function(aItemNode, aSkipAnimation) {
    // XXX: hack to prevent the item details panel to be displayed
    // empty when the user opens the panel very quickly after applying
    // a filter or opening the browser
    new Timer(function() {
      Observers.notify(
        aItemNode, Topics.ITEM_SHOW_DETAILS,
        aSkipAnimation);
    }, 100, Timer.TYPE_ONE_SHOT);
  },

  /**
   * Gets the formatted state of the item.
   * @param aItemState the item state.
   * @param aAlertType type of alert, null if not an alert.
   * @param aEndingSoon whether the item is ending soon or not
   * @return the item state.
   */
  getItemState : function(aItemState, aAlertType, aItem) {

    let stringBundle = Constants.stringBundle;
    let stateLabel = "";
    let stateProperty = null;

    if (aAlertType != null && aAlertType != undefined) {
      switch (aAlertType) {
        case Alert.ALERT_TYPE_BIDDING_ENDING_SOON:
          stateProperty = "bidding.endingSoon";
          break;
        case Alert.ALERT_TYPE_BIDDING_OUTBID:
          stateProperty = "bidding.outbid";
          break;
        case Alert.ALERT_TYPE_BIDDING_HIGH_BIDDER:
          stateProperty = "bidding.highBidder";
          break;
        case Alert.ALERT_TYPE_BIDDING_RAISED_BID:
          stateProperty = "bidding.raisedBid";
          break;
        case Alert.ALERT_TYPE_BIDDING_ITEM_WON:
          stateProperty = "bidding.itemWon";
          break;
        case Alert.ALERT_TYPE_BIDDING_ITEM_LOST:
          stateProperty = "bidding.itemLost";
          break;
        case Alert.ALERT_TYPE_WATCHING_ENDING_SOON:
          stateProperty = "watching.endingSoon";
          break;
        case Alert.ALERT_TYPE_SELLING_ENDING_SOON:
          stateProperty = "selling.endingSoon";
          break;
        case Alert.ALERT_TYPE_SELLING_ITEM_SOLD:
          stateProperty = "selling.itemSold";
          break;
        case Alert.ALERT_TYPE_SELLING_ITEM_UNSOLD:
          stateProperty = "selling.itemUnsold";
          break;
        case Alert.ALERT_TYPE_SELLING_BID_PLACED:
          stateProperty = "selling.bidPlaced";
          break;
        case Alert.ALERT_TYPE_SELLING_RESERVE_MET:
          stateProperty = "selling.reserveMet";
          break;
        case Alert.ALERT_TYPE_BIDDING_BESTOFFER_DECLINED:
          stateProperty = "bidding.bestOffer.declined";
          break;
        case Alert.ALERT_TYPE_BIDDING_BESTOFFER_EXPIRED:
          stateProperty = "bidding.bestOffer.expired";
          break;
        case Alert.ALERT_TYPE_BIDDING_BESTOFFER_COUNTERED:
          stateProperty = "bidding.bestOffer.countered";
          break;
        case Alert.ALERT_TYPE_BIDDING_BESTOFFER_ITEM_WON:
          stateProperty = "bidding.bestOffer.itemWon";
        break;
        case Alert.ALERT_TYPE_SELLING_BESTOFFER_ITEM_SOLD:
          stateProperty = "selling.bestOffer.itemSold";
        break;
        case Alert.ALERT_TYPE_SELLING_BESTOFFER_NEWOFFER:
          stateProperty = "selling.bestOffer.newOffer";
        break;
      }
      if (stateProperty) {
        stateLabel =
          stringBundle.getString("alert." + stateProperty);
      }
      //XXX: There is not alert for best offer pending but if the item
      //state gets updated and the alert is already opened, we should
      //update its state to the right one.
      /* XXX: state not present in Item class for Firefox codebase
      if (aItemState ==
          Item.ITEM_STATE_BEST_OFFER_PENDING) {
        stateProperty = "buying.bestOffer.pending";
        stateLabel =
          stringBundle.getString("item.state." + stateProperty);
      }
      */
    } else {
      switch (aItemState) {
        case Item.ITEM_STATE_WATCHING:
          if (aItem.get("isEnded")) {
            stateProperty = "buying.watching.ended";
          } else {
            stateProperty = "buying.watching";
          }
          break;
        case Item.ITEM_STATE_BUYING_ITEM_WON:
          stateProperty = "buying.won";
          break;
        case Item.ITEM_STATE_BUYING_ITEM_LOST:
          stateProperty = "buying.lost";
          break;
        case Item.
          ITEM_STATE_BUYING_ITEM_LOST_RESERVE_NOT_MET:
          stateProperty = "buying.lost.reserve";
          break;
        case Item.ITEM_STATE_BUYING_SUCCESS:
          stateProperty = "buying.success";
          break;
        case Item.ITEM_STATE_BUYING_OUTBID:
          stateProperty = "buying.outbid";
          break;
        case Item.ITEM_STATE_BUYING_RESERVE_NOT_MET:
          stateProperty = "buying.noReserve";
          break;
        case Item.ITEM_STATE_SELLING:
          stateProperty = "selling";
          break;
        case Item.ITEM_STATE_SELLING_SUCCESS:
          stateProperty = "selling.success";
          break;
        case Item.ITEM_STATE_SELLING_ITEM_SOLD:
          stateProperty = "selling.sold";
          break;
        case Item.ITEM_STATE_SELLING_ITEM_UNSOLD:
          stateProperty = "selling.unsold";
          break;
        case Item.
          ITEM_STATE_SELLING_ITEM_UNSOLD_RESERVE_NOT_MET:
          stateProperty = "selling.unsold.reserve";
          break;
        case Item.ITEM_STATE_SELLING_RESERVE_NOT_MET:
          stateProperty = "selling.noReserve";
          break;
        case Item.ITEM_STATE_WATCHING_CLASSIFIED_AD:
          stateProperty = "buying.watching.classified";
          break;
        case Item.ITEM_STATE_SELLING_CLASSIFIED_AD:
          stateProperty = "selling.classified";
          break;
        case Item.ITEM_STATE_WATCHING_BEST_OFFER:
          stateProperty = "buying.watching.bestOffer";
          break;
        case Item.ITEM_STATE_SELLING_BEST_OFFER:
          stateProperty = "selling.bestOffer";
          break;
        case Item.ITEM_STATE_SELLING_WITH_OFFERS:
          stateProperty = "selling.bestOffer.with.offers";
          break;
        case Item.ITEM_STATE_BEST_OFFER_PENDING:
          stateProperty = "buying.bestOffer.pending";
          break;
        case Item.ITEM_STATE_BEST_OFFER_DECLINED:
          stateProperty = "buying.bestOffer.declined";
          break;
        case Item.ITEM_STATE_BEST_OFFER_EXPIRED:
          stateProperty = "buying.bestOffer.expired";
          break;
        case Item.ITEM_STATE_BEST_OFFER_COUNTERED:
          stateProperty = "buying.bestOffer.countered";
          break;
        case Item.ITEM_STATE_BEST_OFFER_ITEM_WON:
          stateProperty = "buying.bestOffer.itemWon";
        break;
        case Item.ITEM_STATE_BEST_OFFER_ITEM_SOLD:
          stateProperty = "selling.bestOffer.itemSold";
        break;
      }

      if (stateProperty) {
        stateLabel =
          stringBundle.getString("item.state." + stateProperty);
      }
    }
    return stateLabel;

  },

  /**
   *
   */
  getItemTimeLeft : function(aItem) {
    let stringBundle = Constants.stringBundle;
    let timeLabel = "";

    if (!aItem.get("isEnded")) {
      let endTime = aItem.get("endTime");
      let ebayTime = Datasource.getEbayTime().getTime();
      let timeLeft = Math.max(0, endTime - ebayTime);
      if (timeLeft < 1000) {
        timeLabel = stringBundle.getString("item.timeleft.ended.label");
      } else {
        timeLabel = Constants.timeLeftAsString(timeLeft);
      }
    }

    return timeLabel;
  },

  /**
   *
   */
  getIsItemEndingSoon : function(aItem) {
    let endingSoon = false;

    if (!aItem.get("isEnded")) {
      let endTime = aItem.get("endTime");
      let ebayTime = Datasource.getEbayTime().getTime();
      let timeLeft = Math.max(0, endTime - ebayTime);
      let minutesLeft = timeLeft / 1000 / 60;

      if (0 < minutesLeft && minutesLeft < 20) {
        endingSoon = true;
      }
    }

    return endingSoon;
  },

  /**
   * Gets item state color.
   * @param aItemState the item state.
   * @param aEndingSoon whether the item is ending soon or not
   * @return the item state color.
   */
  getItemStateStyle : function(aItemState, aEndingSoon) {
    let stateStyle = null;

    switch (aItemState) {
      case Item.ITEM_STATE_BUYING_ITEM_WON:
      case Item.ITEM_STATE_BUYING_SUCCESS:
      case Item.ITEM_STATE_SELLING_SUCCESS:
      case Item.ITEM_STATE_SELLING_ITEM_SOLD:
      case Item.ITEM_STATE_BEST_OFFER_ITEM_WON:
      case Item.ITEM_STATE_BEST_OFFER_ITEM_SOLD:
      case Item.ITEM_STATE_SELLING_WITH_OFFERS:
      case Item.ITEM_STATE_BEST_OFFER_PENDING:
      case Item.ITEM_STATE_BEST_OFFER_COUNTERED:
        stateStyle = "won";
        break;
      case Item.ITEM_STATE_BUYING_ITEM_LOST:
      case Item.ITEM_STATE_BUYING_ITEM_LOST_RESERVE_NOT_MET:
      case Item.ITEM_STATE_BUYING_OUTBID:
      case Item.ITEM_STATE_BUYING_RESERVE_NOT_MET:
      case Item.ITEM_STATE_SELLING_ITEM_UNSOLD:
      case Item.ITEM_STATE_SELLING_ITEM_UNSOLD_RESERVE_NOT_MET:
      case Item.ITEM_STATE_SELLING_RESERVE_NOT_MET:
      case Item.ITEM_STATE_BEST_OFFER_EXPIRED:
      case Item.ITEM_STATE_BEST_OFFER_DECLINED:
        stateStyle = "outbid";
        break;
      default:
        if (aEndingSoon) {
          stateStyle = "ending-soon";
        } else {
          stateStyle = "watching";
        }
        break;
    }

    return stateStyle;
  }

};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(ItemService);
