/**
 * Copyright (C) 2007-2013 eBay Inc. All Rights Reserved.
 */

var EXPORTED_SYMBOLS = ["Migrations"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

// The format version of the database that is expected
const SCHEMA_VERSION = 13;

var Migrations = {

  /**
   * Initializes the resource.
   */
  init : function() {
    Cu.import("resource://ebaycompanion/helpers/logger.js");
  },

  /**
   * Migrates the database to the current schema
   */
  performMigrations : function() {
    // make the import when we actually need it
    if (typeof(PropertiesStorage) == "undefined") {
      Cu.import("resource://ebaycompanion/storage/propertiesStorage.js");
    }
    let schemaVersion = PropertiesStorage.get("schemaVersion");

    if (schemaVersion) {
      if (schemaVersion > SCHEMA_VERSION) {
        Logger.warning("Database schema version is higher than expected. " +
                       "It's probably been modified by a later version. " +
                       "We'll use it, but you may get errors.");
        return;
      }
      if (typeof(StorageHelper) == "undefined") {
       Cu.import("resource://ebaycompanion/helpers/storageHelper.js");
      }
      while (schemaVersion < SCHEMA_VERSION) {
        schemaVersion++;
        this["schema_" + schemaVersion]();
      }
    }

    PropertiesStorage.set("schemaVersion", SCHEMA_VERSION);
  },

  /**
   * Schema migrations
   */
  schema_2 : function() {
    try {
      StorageHelper.
        doStatement("ALTER TABLE items ADD COLUMN userQuantityWinning DEFAULT 0");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_3 : function() {
    try {
      StorageHelper.
        doStatement("ALTER TABLE items ADD COLUMN thumbnailUrl DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_4 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN sellerFeedbackPercent DEFAULT 0");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_5 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN shippingTermsInDescription DEFAULT false");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN shipToLocations DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN shippingType DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_6 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN highBidderId DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN highBidderFeedbackScore DEFAULT 0");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_7 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN hitCounterType DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_8 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN convertedCurrentPrice DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN convertedCurrentPriceCurrency DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN convertedMaxBid DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN convertedMaxBidCurrency DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_9 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN quantity DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN type DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_10 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE accounts ADD COLUMN email DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_11 : function() {
    try {
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN bestOfferEnabled DEFAULT false");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN bestOffer DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN bestOfferCurrency DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN bestOfferStatus DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN convertedBestOffer DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN convertedBestOfferCurrency DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN bestOfferCount DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE transactions ADD COLUMN transactionPrice DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE transactions ADD COLUMN transactionPriceCurrency DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE transactions ADD COLUMN convertedTransactionPrice DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE transactions ADD COLUMN convertedTransactionPriceCurrency DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_12 : function() {
    try {
      if (typeof(Constants) == "undefined") {
        Cu.import("resource://ebaycompanion/constants.js");
      }
      Constants.prefBranch.set("update.collection", true);
      Logger.warning("Set flag to update the entire collection");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN viewItemUrl DEFAULT ''");
    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_13 : function() {
    try {
      if (typeof(Constants) == "undefined") {
        Cu.import("resource://ebaycompanion/constants.js");
      }
      Constants.prefBranch.set("update.collection", true);
      Logger.warning("Set flag to update the entire collection");

      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN returnsInfo DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN condition DEFAULT 0");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN conditionDisplayName DEFAULT ''");
      StorageHelper.
        doStatement(
          "ALTER TABLE items ADD COLUMN paymentInfo DEFAULT ''");

    } catch (e) {
      Logger.exception(e, Logger.DUMP_STACK)
    }
  },

  schema_14 : function() {
  }
};

/**
 * Constructor.
 */
(function() { this.init(); }).apply(Migrations);
